searchData={"content_type":"text/markdown","items":[{"doc":"Boruta is the core of an OAuth/OpenID Connect provider implementing authorization business rules.\n\n[Documentation](README.md)","ref":"Boruta.html","title":"Boruta","type":"module"},{"doc":"Utilities to access Boruta configuration ad set defaults.\n\nBoruta configuration can be set as following in `config.exs` overriding following default configuration\n```\nconfig :boruta, Boruta.Oauth,\n  repo: MyApp.Repo, # mandatory\n  cache_backend: Boruta.Cache,\n  contexts: [\n    access_tokens: Boruta.Ecto.AccessTokens,\n    clients: Boruta.Ecto.Clients,\n    codes: Boruta.Ecto.Codes,\n    resource_owners: MyApp.ResourceOwners, # mandatory for user flows\n    scopes: Boruta.Ecto.Scopes\n  ],\n  max_ttl: [\n    authorization_code: 60,\n    access_token: 60 * 60 * 24,\n    id_token: 60 * 60 * 24,\n    refresh_token: 60 * 60 * 24 * 30\n  ],\n  token_generator: Boruta.TokenGenerator,\n  issuer: \"boruta\"\n```","ref":"Boruta.Config.html","title":"Boruta.Config","type":"module"},{"doc":"Administration of Boruta resources.","ref":"Boruta.Ecto.Admin.html","title":"Boruta.Ecto.Admin","type":"module"},{"doc":"","ref":"Boruta.Ecto.Admin.html#create_client/1","title":"Boruta.Ecto.Admin.create_client/1","type":"function"},{"doc":"","ref":"Boruta.Ecto.Admin.html#create_scope/1","title":"Boruta.Ecto.Admin.create_scope/1","type":"function"},{"doc":"","ref":"Boruta.Ecto.Admin.html#delete_client/1","title":"Boruta.Ecto.Admin.delete_client/1","type":"function"},{"doc":"","ref":"Boruta.Ecto.Admin.html#delete_inactive_tokens/0","title":"Boruta.Ecto.Admin.delete_inactive_tokens/0","type":"function"},{"doc":"","ref":"Boruta.Ecto.Admin.html#delete_inactive_tokens/1","title":"Boruta.Ecto.Admin.delete_inactive_tokens/1","type":"function"},{"doc":"","ref":"Boruta.Ecto.Admin.html#delete_scope/1","title":"Boruta.Ecto.Admin.delete_scope/1","type":"function"},{"doc":"","ref":"Boruta.Ecto.Admin.html#get_client!/1","title":"Boruta.Ecto.Admin.get_client!/1","type":"function"},{"doc":"","ref":"Boruta.Ecto.Admin.html#get_scope!/1","title":"Boruta.Ecto.Admin.get_scope!/1","type":"function"},{"doc":"","ref":"Boruta.Ecto.Admin.html#get_scopes_by_ids/1","title":"Boruta.Ecto.Admin.get_scopes_by_ids/1","type":"function"},{"doc":"","ref":"Boruta.Ecto.Admin.html#get_scopes_by_names/1","title":"Boruta.Ecto.Admin.get_scopes_by_names/1","type":"function"},{"doc":"","ref":"Boruta.Ecto.Admin.html#list_active_tokens/0","title":"Boruta.Ecto.Admin.list_active_tokens/0","type":"function"},{"doc":"","ref":"Boruta.Ecto.Admin.html#list_active_tokens/1","title":"Boruta.Ecto.Admin.list_active_tokens/1","type":"function"},{"doc":"","ref":"Boruta.Ecto.Admin.html#list_clients/0","title":"Boruta.Ecto.Admin.list_clients/0","type":"function"},{"doc":"","ref":"Boruta.Ecto.Admin.html#list_scopes/0","title":"Boruta.Ecto.Admin.list_scopes/0","type":"function"},{"doc":"","ref":"Boruta.Ecto.Admin.html#regenerate_client_key_pair/1","title":"Boruta.Ecto.Admin.regenerate_client_key_pair/1","type":"function"},{"doc":"","ref":"Boruta.Ecto.Admin.html#regenerate_client_key_pair/3","title":"Boruta.Ecto.Admin.regenerate_client_key_pair/3","type":"function"},{"doc":"","ref":"Boruta.Ecto.Admin.html#regenerate_client_secret/1","title":"Boruta.Ecto.Admin.regenerate_client_secret/1","type":"function"},{"doc":"","ref":"Boruta.Ecto.Admin.html#regenerate_client_secret/2","title":"Boruta.Ecto.Admin.regenerate_client_secret/2","type":"function"},{"doc":"","ref":"Boruta.Ecto.Admin.html#update_client/2","title":"Boruta.Ecto.Admin.update_client/2","type":"function"},{"doc":"","ref":"Boruta.Ecto.Admin.html#update_scope/2","title":"Boruta.Ecto.Admin.update_scope/2","type":"function"},{"doc":"OAuth requests entrypoint, provides authorization artifacts to clients as stated in [RFC](https://datatracker.ietf.org/doc/html/rfc6749#section-4).\n\n> __Note__: this module follows inverted heaxagonal architecture, its functions will invoke callbacks of the given module argument and return its result.\n>\n> The definition of those callbacks are provided by either `Boruta.Oauth.Application` or `Boruta.Oauth.AuthorizeApplication`, `Boruta.Oauth.TokenApplication`, `Boruta.Oauth.IntrospectApplication`, and `Boruta.Oauth.RevokeApplication`,","ref":"Boruta.Oauth.html","title":"Boruta.Oauth","type":"module"},{"doc":"Process an authorize request as stated in [RFC 6749 - The OAuth 2.0 Authorization Framework](https://tools.ietf.org/html/rfc6749) and [OpenID Connect Core 1.0 incorporating errata set 1](https://openid.net/specs/openid-connect-core-1_0.html#HybridFlowAuth).\n\nTriggers `authorize_success` in case of success and `authorize_error` in case of failure from the given `module`. Those functions are described in `Boruta.Oauth.Application` behaviour.","ref":"Boruta.Oauth.html#authorize/3","title":"Boruta.Oauth.authorize/3","type":"function"},{"doc":"Process a introspect request as stated in [RFC 7662 - OAuth 2.0 Token Introspection](https://tools.ietf.org/html/rfc7662).\n\nTriggers `introspect_success` in case of success and `introspect_error` in case of failure from the given `module`. Those functions are described in `Boruta.Oauth.Application` behaviour.","ref":"Boruta.Oauth.html#introspect/2","title":"Boruta.Oauth.introspect/2","type":"function"},{"doc":"Check success of an authorize request as stated in [RFC 6749 - The OAuth 2.0 Authorization Framework](https://tools.ietf.org/html/rfc6749) and [OpenID Connect Core 1.0 incorporating errata set 1](https://openid.net/specs/openid-connect-core-1_0.html#HybridFlowAuth).\n\nTriggers `preauthorize_success` in case of success and `preauthorize_error` in case of failure from the given `module`. Those functions are described in `Boruta.Oauth.Application` behaviour.","ref":"Boruta.Oauth.html#preauthorize/3","title":"Boruta.Oauth.preauthorize/3","type":"function"},{"doc":"Process a revoke request as stated in [RFC 7009 - OAuth 2.0 Token Revocation](https://tools.ietf.org/html/rfc7009).\n\nTriggers `revoke_success` in case of success and `revoke_error` in case of failure from the given `module`. Those functions are described in `Boruta.Oauth.Application` behaviour.","ref":"Boruta.Oauth.html#revoke/2","title":"Boruta.Oauth.revoke/2","type":"function"},{"doc":"Process an token request as stated in [RFC 6749 - The OAuth 2.0 Authorization Framework](https://tools.ietf.org/html/rfc6749).\n\nTriggers `token_success` in case of success and `token_error` in case of failure from the given `module`. Those functions are described in `Boruta.Oauth.Application` behaviour.","ref":"Boruta.Oauth.html#token/2","title":"Boruta.Oauth.token/2","type":"function"},{"doc":"Openid requests entrypoint, provides additional artifacts to OAuth as stated in [Openid Connect Core 1.0](https://openid.net/specs/openid-connect-core-1_0.html).\n\n> __Note__: this module follows inverted heaxagonal architecture, its functions will invoke callbacks of the given module argument and return its result.\n>\n> The definition of those callbacks are provided by either `Boruta.Openid.Application` or `Boruta.Openid.JwksApplication` and `Boruta.Openid.UserinfoApplication`","ref":"Boruta.Openid.html","title":"Boruta.Openid","type":"module"},{"doc":"","ref":"Boruta.Openid.html#jwks/2","title":"Boruta.Openid.jwks/2","type":"function"},{"doc":"","ref":"Boruta.Openid.html#register_client/3","title":"Boruta.Openid.register_client/3","type":"function"},{"doc":"","ref":"Boruta.Openid.html#userinfo/2","title":"Boruta.Openid.userinfo/2","type":"function"},{"doc":"This task will help creation of a basic OAuth/OpenID Connect server by providing needed phoenix controllers, views and templates to expose OAuth endpoints.\n\nControllers are unit tested using Mox, you'll need to add that dependency in order to run them (see below).","ref":"Mix.Tasks.Boruta.Gen.Controllers.html","title":"mix boruta.gen.controllers","type":"task"},{"doc":"```\nmix boruta.gen.controllers\n```","ref":"Mix.Tasks.Boruta.Gen.Controllers.html#module-examples","title":"Examples - mix boruta.gen.controllers","type":"task"},{"doc":"* You can add OAuth routes in web application router as follow to expose controller actions\n\n```elixir\nscope \"/oauth\", MyAppWeb.Oauth do\n  pipe_through :api\n\n  post \"/revoke\", RevokeController, :revoke\n  post \"/token\", TokenController, :token\n  post \"/introspect\", IntrospectController, :introspect\nend\n\nscope \"/openid\", MyAppWeb.Openid do\n  pipe_through [:api]\n\n  get \"/userinfo\", UserinfoController, :userinfo\n  post \"/userinfo\", UserinfoController, :userinfo\n  get \"/jwks\", JwksController, :jwks_index\nend\n\n####\n\nscope \"/oauth\", MyAppWeb.Oauth do\n  pipe_through [:browser, :fetch_current_user]\n\n  get \"/authorize\", AuthorizeController, :authorize\nend","ref":"Mix.Tasks.Boruta.Gen.Controllers.html#module-post-installation-steps","title":"Post installation steps - mix boruta.gen.controllers","type":"task"},{"doc":"scope \"/openid\", MyAppWeb.Openid do\n  pipe_through [:browser, :fetch_current_user]\n\n  get \"/authorize\", AuthorizeController, :authorize\nend\n```\n\n#","ref":"Mix.Tasks.Boruta.Gen.Controllers.html#module-or","title":"OR - mix boruta.gen.controllers","type":"task"},{"doc":"* Add mox dependency in order to run controller unit tests\n\n```elixir\n{:mox, \"~> 0.5\", only: :test}\n```\n\n* Add following in config/test.exs\n\n```elixir\nconfig :myapp, :oauth_module, Boruta.OauthMock\nconfig :myapp, :openid_module, Boruta.OpenidMock\n```\n\n* Add following in test/test_helper.exs\n\n```elixir\nMox.defmock(Boruta.OauthMock, for: Boruta.OauthModule)\nMox.defmock(Boruta.OpenidMock, for: Boruta.OpenidModule)\n```\n\n#","ref":"Mix.Tasks.Boruta.Gen.Controllers.html#module-testing","title":"Testing - mix boruta.gen.controllers","type":"task"},{"doc":"All flows involving resource owners need its integration guided by `Boruta.Oauth.ResourceOwners` behaviour.\nFor authorize endpoint, you'll need to assign current_user with a plug and setup login redirections which, with raw setup, raise an error where it is required.","ref":"Mix.Tasks.Boruta.Gen.Controllers.html#module-user-flows","title":"User flows - mix boruta.gen.controllers","type":"task"},{"doc":"","ref":"Mix.Tasks.Boruta.Gen.Controllers.html#run/1","title":"Mix.Tasks.Boruta.Gen.Controllers.run/1","type":"function"},{"doc":"Migration task for Boruta.\n\nCreates `oauth_clients`, `oauth_scopes` and `oauth_tokens` tables.\n\n> __Note__: This task will create migration files to keep your integration up to date by checking migration file and module names to create missing ones.\n> Thus it is not recommended to change naming since it will recreate the migration in further runs.","ref":"Mix.Tasks.Boruta.Gen.Migration.html","title":"mix boruta.gen.migration","type":"task"},{"doc":"This task will trigger a REPL to gather statistical info from your usage of the library. The owners are thankful to you for providing those informations since it helps better maintain the product.","ref":"Mix.Tasks.Boruta.Gen.Migration.html#module-usage-statistics-gathering","title":"Usage statistics gathering - mix boruta.gen.migration","type":"task"},{"doc":"```\nmix boruta.gen.migration\n```","ref":"Mix.Tasks.Boruta.Gen.Migration.html#module-examples","title":"Examples - mix boruta.gen.migration","type":"task"},{"doc":"Implement this behaviour in the application layer of your OAuth / OpenID Connect provider.\nThis behaviour gives all callbacks triggered invoking `Boruta.Oauth` module functions.\n\n> __Note__: This behaviour is split into `Boruta.Oauth.AuthorizeApplication`, `Boruta.Oauth.TokenApplication`, `Boruta.Oauth.IntrospectApplication`, and `Boruta.Oauth.RevokeApplication` providing utilities to implement the different OAuth / OpenID Connect endpoints independently.","ref":"Boruta.Oauth.Application.html","title":"Boruta.Oauth.Application","type":"behaviour"},{"doc":"This function will be triggered in case of failure invoking `Boruta.Oauth.authorize/3`","ref":"Boruta.Oauth.Application.html#c:authorize_error/2","title":"Boruta.Oauth.Application.authorize_error/2","type":"callback"},{"doc":"This function will be triggered in case of success invoking `Boruta.Oauth.authorize/3`","ref":"Boruta.Oauth.Application.html#c:authorize_success/2","title":"Boruta.Oauth.Application.authorize_success/2","type":"callback"},{"doc":"This function will be triggered in case of failure invoking `Boruta.Oauth.introspect/2`","ref":"Boruta.Oauth.Application.html#c:introspect_error/2","title":"Boruta.Oauth.Application.introspect_error/2","type":"callback"},{"doc":"This function will be triggered in case of success invoking `Boruta.Oauth.introspect/2`","ref":"Boruta.Oauth.Application.html#c:introspect_success/2","title":"Boruta.Oauth.Application.introspect_success/2","type":"callback"},{"doc":"This function will be triggered in case of failure invoking `Boruta.Oauth.preauthorize/3`","ref":"Boruta.Oauth.Application.html#c:preauthorize_error/2","title":"Boruta.Oauth.Application.preauthorize_error/2","type":"callback"},{"doc":"This function will be triggered in case of success invoking `Boruta.Oauth.preauthorize/3`","ref":"Boruta.Oauth.Application.html#c:preauthorize_success/2","title":"Boruta.Oauth.Application.preauthorize_success/2","type":"callback"},{"doc":"This function will be triggered in case of failure invoking `Boruta.Oauth.revoke/2`","ref":"Boruta.Oauth.Application.html#c:revoke_error/2","title":"Boruta.Oauth.Application.revoke_error/2","type":"callback"},{"doc":"This function will be triggered in case of success invoking `Boruta.Oauth.revoke/2`","ref":"Boruta.Oauth.Application.html#c:revoke_success/1","title":"Boruta.Oauth.Application.revoke_success/1","type":"callback"},{"doc":"This function will be triggered in case of failure invoking `Boruta.Oauth.token/2`","ref":"Boruta.Oauth.Application.html#c:token_error/2","title":"Boruta.Oauth.Application.token_error/2","type":"callback"},{"doc":"This function will be triggered in case of success invoking `Boruta.Oauth.token/2`","ref":"Boruta.Oauth.Application.html#c:token_success/2","title":"Boruta.Oauth.Application.token_success/2","type":"callback"},{"doc":"OAuth application behaviour - authorize endpoint\n\nImplement this behaviour in the application layer of your OAuth provider. The callbacks are triggered while calling functions from `Boruta.Oauth` module.","ref":"Boruta.Oauth.AuthorizeApplication.html","title":"Boruta.Oauth.AuthorizeApplication","type":"behaviour"},{"doc":"This function will be triggered in case of failure invoking `Boruta.Oauth.authorize/3`","ref":"Boruta.Oauth.AuthorizeApplication.html#c:authorize_error/2","title":"Boruta.Oauth.AuthorizeApplication.authorize_error/2","type":"callback"},{"doc":"This function will be triggered in case of success invoking `Boruta.Oauth.authorize/3`","ref":"Boruta.Oauth.AuthorizeApplication.html#c:authorize_success/2","title":"Boruta.Oauth.AuthorizeApplication.authorize_success/2","type":"callback"},{"doc":"This function will be triggered in case of failure invoking `Boruta.Oauth.preauthorize/3`","ref":"Boruta.Oauth.AuthorizeApplication.html#c:preauthorize_error/2","title":"Boruta.Oauth.AuthorizeApplication.preauthorize_error/2","type":"callback"},{"doc":"This function will be triggered in case of success invoking `Boruta.Oauth.preauthorize/3`","ref":"Boruta.Oauth.AuthorizeApplication.html#c:preauthorize_success/2","title":"Boruta.Oauth.AuthorizeApplication.preauthorize_success/2","type":"callback"},{"doc":"OAuth application behaviour - introspect endpoint\n\nImplement this behaviour in the application layer of your OAuth provider. The callbacks are triggered while calling functions from `Boruta.Oauth` module.","ref":"Boruta.Oauth.IntrospectApplication.html","title":"Boruta.Oauth.IntrospectApplication","type":"behaviour"},{"doc":"This function will be triggered in case of failure invoking `Boruta.Oauth.introspect/2`","ref":"Boruta.Oauth.IntrospectApplication.html#c:introspect_error/2","title":"Boruta.Oauth.IntrospectApplication.introspect_error/2","type":"callback"},{"doc":"This function will be triggered in case of success invoking `Boruta.Oauth.introspect/2`","ref":"Boruta.Oauth.IntrospectApplication.html#c:introspect_success/2","title":"Boruta.Oauth.IntrospectApplication.introspect_success/2","type":"callback"},{"doc":"OAuth application behaviour - revoke endpoint\n\nImplement this behaviour in the application layer of your OAuth provider. The callbacks are triggered while calling functions from `Boruta.Oauth` module.","ref":"Boruta.Oauth.RevokeApplication.html","title":"Boruta.Oauth.RevokeApplication","type":"behaviour"},{"doc":"This function will be triggered in case of failure invoking `Boruta.Oauth.revoke/2`","ref":"Boruta.Oauth.RevokeApplication.html#c:revoke_error/2","title":"Boruta.Oauth.RevokeApplication.revoke_error/2","type":"callback"},{"doc":"This function will be triggered in case of success invoking `Boruta.Oauth.revoke/2`","ref":"Boruta.Oauth.RevokeApplication.html#c:revoke_success/1","title":"Boruta.Oauth.RevokeApplication.revoke_success/1","type":"callback"},{"doc":"OAuth application behaviour - token endpoint\n\nImplement this behaviour in the application layer of your OAuth provider. The callbacks are triggered while calling functions from `Boruta.Oauth` module.","ref":"Boruta.Oauth.TokenApplication.html","title":"Boruta.Oauth.TokenApplication","type":"behaviour"},{"doc":"This function will be triggered in case of failure invoking `Boruta.Oauth.token/2`","ref":"Boruta.Oauth.TokenApplication.html#c:token_error/2","title":"Boruta.Oauth.TokenApplication.token_error/2","type":"callback"},{"doc":"This function will be triggered in case of success invoking `Boruta.Oauth.token/2`","ref":"Boruta.Oauth.TokenApplication.html#c:token_success/2","title":"Boruta.Oauth.TokenApplication.token_success/2","type":"callback"},{"doc":"Implement this behaviour in the application layer of your OpenID Connect provider.\nThis behaviour gives all callbacks triggered invoking `Boruta.Openid` module functions.\n\n> __Note__: This behaviour is split into `Boruta.Openid.JwksApplication` and `Boruta.Openid.UserinfoApplication` providing utilities to implement the different OpenID Connect endpoints independently.","ref":"Boruta.Openid.Application.html","title":"Boruta.Openid.Application","type":"behaviour"},{"doc":"This function will be triggered when request is unauthorized invoking `Boruta.Openid.register_client/3`","ref":"Boruta.Openid.Application.html#c:client_registered/2","title":"Boruta.Openid.Application.client_registered/2","type":"callback"},{"doc":"This function will be triggered in case of success invoking `Boruta.Openid.jwks/2`","ref":"Boruta.Openid.Application.html#c:jwk_list/2","title":"Boruta.Openid.Application.jwk_list/2","type":"callback"},{"doc":"This function will be triggered when request is unauthorized invoking `Boruta.Openid.register_client/3`","ref":"Boruta.Openid.Application.html#c:registration_failure/2","title":"Boruta.Openid.Application.registration_failure/2","type":"callback"},{"doc":"This function will be triggered when request is unauthorized invoking `Boruta.Openid.userinfo/2`","ref":"Boruta.Openid.Application.html#c:unauthorized/2","title":"Boruta.Openid.Application.unauthorized/2","type":"callback"},{"doc":"This function will be triggered in case of success invoking `Boruta.Openid.userinfo/2`","ref":"Boruta.Openid.Application.html#c:userinfo_fetched/2","title":"Boruta.Openid.Application.userinfo_fetched/2","type":"callback"},{"doc":"Implement this behaviour in the application layer of your OpenID Connect provider.\nThis behaviour gives all callbacks triggered invoking `Boruta.Openid.register_client/3` function.","ref":"Boruta.Openid.DynamicRegistrationApplication.html","title":"Boruta.Openid.DynamicRegistrationApplication","type":"behaviour"},{"doc":"This function will be triggered when request is unauthorized invoking `Boruta.Openid.register_client/3`","ref":"Boruta.Openid.DynamicRegistrationApplication.html#c:client_registered/2","title":"Boruta.Openid.DynamicRegistrationApplication.client_registered/2","type":"callback"},{"doc":"This function will be triggered when request is unauthorized invoking `Boruta.Openid.register_client/3`","ref":"Boruta.Openid.DynamicRegistrationApplication.html#c:registration_failure/2","title":"Boruta.Openid.DynamicRegistrationApplication.registration_failure/2","type":"callback"},{"doc":"Implement this behaviour in the application layer of your OpenID Connect provider.\nThis behaviour gives all callbacks triggered invoking `Boruta.Openid.jwks/2` function.","ref":"Boruta.Openid.JwksApplication.html","title":"Boruta.Openid.JwksApplication","type":"behaviour"},{"doc":"This function will be triggered in case of success invoking `Boruta.Opeind.jwks/2`","ref":"Boruta.Openid.JwksApplication.html#c:jwk_list/2","title":"Boruta.Openid.JwksApplication.jwk_list/2","type":"callback"},{"doc":"Implement this behaviour in the application layer of your OpenID Connect provider.\nThis behaviour gives all callbacks triggered invoking `Boruta.Openid.userinfo/2` function.","ref":"Boruta.Openid.UserinfoApplication.html","title":"Boruta.Openid.UserinfoApplication","type":"behaviour"},{"doc":"This function will be triggered when request is unauthorized invoking `Boruta.Openid.userinfo/2`","ref":"Boruta.Openid.UserinfoApplication.html#c:unauthorized/2","title":"Boruta.Openid.UserinfoApplication.unauthorized/2","type":"callback"},{"doc":"This function will be triggered in case of success invoking `Boruta.Openid.userinfo/2`","ref":"Boruta.Openid.UserinfoApplication.html#c:userinfo_fetched/2","title":"Boruta.Openid.UserinfoApplication.userinfo_fetched/2","type":"callback"},{"doc":"Response returned in case of authorization request success. Provides utilities and mandatory data needed to respond to the authorize part of implicit, code and hybrid flows.","ref":"Boruta.Oauth.AuthorizeResponse.html","title":"Boruta.Oauth.AuthorizeResponse","type":"module"},{"doc":"","ref":"Boruta.Oauth.AuthorizeResponse.html#from_tokens/2","title":"Boruta.Oauth.AuthorizeResponse.from_tokens/2","type":"function"},{"doc":"","ref":"Boruta.Oauth.AuthorizeResponse.html#redirect_to_url/1","title":"Boruta.Oauth.AuthorizeResponse.redirect_to_url/1","type":"function"},{"doc":"","ref":"Boruta.Oauth.AuthorizeResponse.html#t:t/0","title":"Boruta.Oauth.AuthorizeResponse.t/0","type":"type"},{"doc":"Response returned in case of introspection request success. Provides mandatory data needed to respond to token introspection.","ref":"Boruta.Oauth.IntrospectResponse.html","title":"Boruta.Oauth.IntrospectResponse","type":"module"},{"doc":"","ref":"Boruta.Oauth.IntrospectResponse.html#from_error/1","title":"Boruta.Oauth.IntrospectResponse.from_error/1","type":"function"},{"doc":"","ref":"Boruta.Oauth.IntrospectResponse.html#from_token/1","title":"Boruta.Oauth.IntrospectResponse.from_token/1","type":"function"},{"doc":"","ref":"Boruta.Oauth.IntrospectResponse.html#t:t/0","title":"Boruta.Oauth.IntrospectResponse.t/0","type":"type"},{"doc":"Response returned in case of access token request success. Provides utilities and mandatory data needed to respond to the token part of client credentials, resource owner password, code and hybrid flows.","ref":"Boruta.Oauth.TokenResponse.html","title":"Boruta.Oauth.TokenResponse","type":"module"},{"doc":"","ref":"Boruta.Oauth.TokenResponse.html#from_token/1","title":"Boruta.Oauth.TokenResponse.from_token/1","type":"function"},{"doc":"","ref":"Boruta.Oauth.TokenResponse.html#t:t/0","title":"Boruta.Oauth.TokenResponse.t/0","type":"type"},{"doc":"`Boruta.Ecto.Client` resource administration","ref":"Boruta.Ecto.Admin.Clients.html","title":"Boruta.Ecto.Admin.Clients","type":"module"},{"doc":"Creates a client.","ref":"Boruta.Ecto.Admin.Clients.html#create_client/1","title":"Boruta.Ecto.Admin.Clients.create_client/1","type":"function"},{"doc":"iex> create_client(%{field: value})\n    {:ok, %Client{}}\n\n    iex> create_client(%{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Boruta.Ecto.Admin.Clients.html#create_client/1-examples","title":"Examples - Boruta.Ecto.Admin.Clients.create_client/1","type":"function"},{"doc":"Deletes a Client.","ref":"Boruta.Ecto.Admin.Clients.html#delete_client/1","title":"Boruta.Ecto.Admin.Clients.delete_client/1","type":"function"},{"doc":"iex> delete_client(client)\n    {:ok, %Client{}}\n\n    iex> delete_client(client)\n    {:error, %Ecto.Changeset{}}","ref":"Boruta.Ecto.Admin.Clients.html#delete_client/1-examples","title":"Examples - Boruta.Ecto.Admin.Clients.delete_client/1","type":"function"},{"doc":"Gets a single client.\n\nRaises `Ecto.NoResultsError` if the Client does not exist.","ref":"Boruta.Ecto.Admin.Clients.html#get_client!/1","title":"Boruta.Ecto.Admin.Clients.get_client!/1","type":"function"},{"doc":"iex> get_client!(123)\n    %Client{}\n\n    iex> get_client!(456)\n    ** (Ecto.NoResultsError)","ref":"Boruta.Ecto.Admin.Clients.html#get_client!/1-examples","title":"Examples - Boruta.Ecto.Admin.Clients.get_client!/1","type":"function"},{"doc":"Returns the list of clients.","ref":"Boruta.Ecto.Admin.Clients.html#list_clients/0","title":"Boruta.Ecto.Admin.Clients.list_clients/0","type":"function"},{"doc":"iex> list_clients()\n    [%Client{}, ...]","ref":"Boruta.Ecto.Admin.Clients.html#list_clients/0-examples","title":"Examples - Boruta.Ecto.Admin.Clients.list_clients/0","type":"function"},{"doc":"Regenerates client key pair. If a couple public/private key is provided as parameter, updates it.","ref":"Boruta.Ecto.Admin.Clients.html#regenerate_client_key_pair/3","title":"Boruta.Ecto.Admin.Clients.regenerate_client_key_pair/3","type":"function"},{"doc":"iex> regenerate_client_key_pair(client)\n    {:ok, %Client{}}","ref":"Boruta.Ecto.Admin.Clients.html#regenerate_client_key_pair/3-examples","title":"Examples - Boruta.Ecto.Admin.Clients.regenerate_client_key_pair/3","type":"function"},{"doc":"Regenerates client secret. If a secret is provided as parameter, updates it.","ref":"Boruta.Ecto.Admin.Clients.html#regenerate_client_secret/2","title":"Boruta.Ecto.Admin.Clients.regenerate_client_secret/2","type":"function"},{"doc":"iex> regenerate_client_secret(client)\n    {:ok, %Client{}}","ref":"Boruta.Ecto.Admin.Clients.html#regenerate_client_secret/2-examples","title":"Examples - Boruta.Ecto.Admin.Clients.regenerate_client_secret/2","type":"function"},{"doc":"Updates a client.","ref":"Boruta.Ecto.Admin.Clients.html#update_client/2","title":"Boruta.Ecto.Admin.Clients.update_client/2","type":"function"},{"doc":"iex> update_client(client, %{field: new_value})\n    {:ok, %Client{}}\n\n    iex> update_client(client, %{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Boruta.Ecto.Admin.Clients.html#update_client/2-examples","title":"Examples - Boruta.Ecto.Admin.Clients.update_client/2","type":"function"},{"doc":"`Boruta.Ecto.Scope` resource administration","ref":"Boruta.Ecto.Admin.Scopes.html","title":"Boruta.Ecto.Admin.Scopes","type":"module"},{"doc":"Creates a scope.","ref":"Boruta.Ecto.Admin.Scopes.html#create_scope/1","title":"Boruta.Ecto.Admin.Scopes.create_scope/1","type":"function"},{"doc":"iex> create_scope(%{field: value})\n    {:ok, %Scope{}}\n\n    iex> create_scope(%{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Boruta.Ecto.Admin.Scopes.html#create_scope/1-examples","title":"Examples - Boruta.Ecto.Admin.Scopes.create_scope/1","type":"function"},{"doc":"Deletes a Scope.","ref":"Boruta.Ecto.Admin.Scopes.html#delete_scope/1","title":"Boruta.Ecto.Admin.Scopes.delete_scope/1","type":"function"},{"doc":"iex> delete_scope(scope)\n    {:ok, %Scope{}}\n\n    iex> delete_scope(scope)\n    {:error, %Ecto.Changeset{}}","ref":"Boruta.Ecto.Admin.Scopes.html#delete_scope/1-examples","title":"Examples - Boruta.Ecto.Admin.Scopes.delete_scope/1","type":"function"},{"doc":"Gets a single scope.\n\nRaises `Ecto.NoResultsError` if the Scope does not exist.","ref":"Boruta.Ecto.Admin.Scopes.html#get_scope!/1","title":"Boruta.Ecto.Admin.Scopes.get_scope!/1","type":"function"},{"doc":"iex> get_scope!(123)\n    %Scope{}\n\n    iex> get_scope!(456)\n    ** (Ecto.NoResultsError)","ref":"Boruta.Ecto.Admin.Scopes.html#get_scope!/1-examples","title":"Examples - Boruta.Ecto.Admin.Scopes.get_scope!/1","type":"function"},{"doc":"Return scopes corresponding to the given ids.","ref":"Boruta.Ecto.Admin.Scopes.html#get_scopes_by_ids/1","title":"Boruta.Ecto.Admin.Scopes.get_scopes_by_ids/1","type":"function"},{"doc":"iex> get_scopes_by_ids([\"9864decf-8aa4-4387-a7e4-19d1a059c912\", \"7b6ec16d-ad8f-4332-ac55-559184ec0f51\"])\n    [%Scope{}, ...]","ref":"Boruta.Ecto.Admin.Scopes.html#get_scopes_by_ids/1-examples","title":"Examples - Boruta.Ecto.Admin.Scopes.get_scopes_by_ids/1","type":"function"},{"doc":"Return scopes corresponding to the given names.","ref":"Boruta.Ecto.Admin.Scopes.html#get_scopes_by_names/1","title":"Boruta.Ecto.Admin.Scopes.get_scopes_by_names/1","type":"function"},{"doc":"iex> get_scopes_by_names([\"first:scope\", \"second:scope\"])\n    [%Scope{}, ...]","ref":"Boruta.Ecto.Admin.Scopes.html#get_scopes_by_names/1-examples","title":"Examples - Boruta.Ecto.Admin.Scopes.get_scopes_by_names/1","type":"function"},{"doc":"Returns the list of scopes.","ref":"Boruta.Ecto.Admin.Scopes.html#list_scopes/0","title":"Boruta.Ecto.Admin.Scopes.list_scopes/0","type":"function"},{"doc":"iex> list_scopes()\n    [%Scope{}, ...]","ref":"Boruta.Ecto.Admin.Scopes.html#list_scopes/0-examples","title":"Examples - Boruta.Ecto.Admin.Scopes.list_scopes/0","type":"function"},{"doc":"Updates a scope.","ref":"Boruta.Ecto.Admin.Scopes.html#update_scope/2","title":"Boruta.Ecto.Admin.Scopes.update_scope/2","type":"function"},{"doc":"iex> update_scope(scope, %{field: new_value})\n    {:ok, %Scope{}}\n\n    iex> update_scope(scope, %{field: bad_value})\n    {:error, %Ecto.Changeset{}}","ref":"Boruta.Ecto.Admin.Scopes.html#update_scope/2-examples","title":"Examples - Boruta.Ecto.Admin.Scopes.update_scope/2","type":"function"},{"doc":"`Boruta.Ecto.Token` resource administration","ref":"Boruta.Ecto.Admin.Tokens.html","title":"Boruta.Ecto.Admin.Tokens","type":"module"},{"doc":"","ref":"Boruta.Ecto.Admin.Tokens.html#delete_inactive_tokens/1","title":"Boruta.Ecto.Admin.Tokens.delete_inactive_tokens/1","type":"function"},{"doc":"Returns all active tokens given `Ecto.Queryable` (defaults to all).","ref":"Boruta.Ecto.Admin.Tokens.html#list_active_tokens/1","title":"Boruta.Ecto.Admin.Tokens.list_active_tokens/1","type":"function"},{"doc":"iex> list_active_tokens()\n    [%Token{}, ...]","ref":"Boruta.Ecto.Admin.Tokens.html#list_active_tokens/1-examples","title":"Examples - Boruta.Ecto.Admin.Tokens.list_active_tokens/1","type":"function"},{"doc":"OAuth requests authorization","ref":"Boruta.Oauth.Authorization.html","title":"Boruta.Oauth.Authorization","type":"protocol"},{"doc":"Checks if request is valid for token creation for given request, depending of implementation.","ref":"Boruta.Oauth.Authorization.html#preauthorize/1","title":"Boruta.Oauth.Authorization.preauthorize/1","type":"function"},{"doc":"Creates and returns tokens for given request, depending of implementation.","ref":"Boruta.Oauth.Authorization.html#token/1","title":"Boruta.Oauth.Authorization.token/1","type":"function"},{"doc":"","ref":"Boruta.Oauth.Authorization.html#t:t/0","title":"Boruta.Oauth.Authorization.t/0","type":"type"},{"doc":"Check against given params and return the corresponding access token","ref":"Boruta.Oauth.Authorization.AccessToken.html","title":"Boruta.Oauth.Authorization.AccessToken","type":"module"},{"doc":"Authorize the access token corresponding to the given params.","ref":"Boruta.Oauth.Authorization.AccessToken.html#authorize/1","title":"Boruta.Oauth.Authorization.AccessToken.authorize/1","type":"function"},{"doc":"iex> authorize(%{value: \"value\"})\n    {:ok, %Boruta.Oauth.Token{...}}","ref":"Boruta.Oauth.Authorization.AccessToken.html#authorize/1-examples","title":"Examples - Boruta.Oauth.Authorization.AccessToken.authorize/1","type":"function"},{"doc":"Check against given params and return the corresponding client","ref":"Boruta.Oauth.Authorization.Client.html","title":"Boruta.Oauth.Authorization.Client","type":"module"},{"doc":"Authorize the client corresponding to the given params.","ref":"Boruta.Oauth.Authorization.Client.html#authorize/1","title":"Boruta.Oauth.Authorization.Client.authorize/1","type":"function"},{"doc":"iex> authorize(id: \"id\", secret: \"secret\")\n    {:ok, %Boruta.Oauth.Client{...}}","ref":"Boruta.Oauth.Authorization.Client.html#authorize/1-examples","title":"Examples - Boruta.Oauth.Authorization.Client.authorize/1","type":"function"},{"doc":"Check against given params and return the corresponding code","ref":"Boruta.Oauth.Authorization.Code.html","title":"Boruta.Oauth.Authorization.Code","type":"module"},{"doc":"Authorize the code corresponding to the given params.","ref":"Boruta.Oauth.Authorization.Code.html#authorize/1","title":"Boruta.Oauth.Authorization.Code.authorize/1","type":"function"},{"doc":"iex> authorize(value: \"value\", redirect_uri: \"redirect_uri\")\n    {:ok, %Boruta.Oauth.Token{...}}","ref":"Boruta.Oauth.Authorization.Code.html#authorize/1-examples","title":"Examples - Boruta.Oauth.Authorization.Code.authorize/1","type":"function"},{"doc":"Check OpenID Connect nonce against given request","ref":"Boruta.Oauth.Authorization.Nonce.html","title":"Boruta.Oauth.Authorization.Nonce","type":"protocol"},{"doc":"Authorize the given request corresponding the nonce value.","ref":"Boruta.Oauth.Authorization.Nonce.html#authorize/1","title":"Boruta.Oauth.Authorization.Nonce.authorize/1","type":"function"},{"doc":"iex> authorize(%CodeRequest{...})\n    :ok","ref":"Boruta.Oauth.Authorization.Nonce.html#authorize/1-examples","title":"Examples - Boruta.Oauth.Authorization.Nonce.authorize/1","type":"function"},{"doc":"","ref":"Boruta.Oauth.Authorization.Nonce.html#t:t/0","title":"Boruta.Oauth.Authorization.Nonce.t/0","type":"type"},{"doc":"Check against given params and return the corresponding resource owner","ref":"Boruta.Oauth.Authorization.ResourceOwner.html","title":"Boruta.Oauth.Authorization.ResourceOwner","type":"module"},{"doc":"Authorize the resource owner corresponding to the given params.","ref":"Boruta.Oauth.Authorization.ResourceOwner.html#authorize/1","title":"Boruta.Oauth.Authorization.ResourceOwner.authorize/1","type":"function"},{"doc":"iex> authorize(id: \"id\")\n    {:ok, %Boruta.Oauth.ResourceOwner{...}}","ref":"Boruta.Oauth.Authorization.ResourceOwner.html#authorize/1-examples","title":"Examples - Boruta.Oauth.Authorization.ResourceOwner.authorize/1","type":"function"},{"doc":"Check against given params and return the corresponding scopes","ref":"Boruta.Oauth.Authorization.Scope.html","title":"Boruta.Oauth.Authorization.Scope","type":"module"},{"doc":"Authorize the given scope according to the given client.","ref":"Boruta.Oauth.Authorization.Scope.html#authorize/1","title":"Boruta.Oauth.Authorization.Scope.authorize/1","type":"function"},{"doc":"iex> authorize(%{scope: \"scope\", client: %Client{...}})\n    {:ok, \"scope\"}","ref":"Boruta.Oauth.Authorization.Scope.html#authorize/1-examples","title":"Examples - Boruta.Oauth.Authorization.Scope.authorize/1","type":"function"},{"doc":"Struct encapsulating an authorization success data","ref":"Boruta.Oauth.AuthorizationSuccess.html","title":"Boruta.Oauth.AuthorizationSuccess","type":"module"},{"doc":"","ref":"Boruta.Oauth.AuthorizationSuccess.html#t:t/0","title":"Boruta.Oauth.AuthorizationSuccess.t/0","type":"type"},{"doc":"Access token introspection","ref":"Boruta.Oauth.Introspect.html","title":"Boruta.Oauth.Introspect","type":"module"},{"doc":"Returns corresponding token for the given `Boruta.Oauth.IntrospectRequest`\n\nNote : Invalid tokens returns an error `{:error, %Error{error: :invalid_access_token, ...}}`. That must be rescued to return `%{\"active\" => false}` in application implementation.","ref":"Boruta.Oauth.Introspect.html#token/1","title":"Boruta.Oauth.Introspect.token/1","type":"function"},{"doc":"iex> token(%IntrospectRequest{\n      client_id: \"client_id\",\n      client_secret: \"client_secret\",\n      token: \"token\"\n    })\n    {:ok, %Token{...}}","ref":"Boruta.Oauth.Introspect.html#token/1-examples","title":"Examples - Boruta.Oauth.Introspect.token/1","type":"function"},{"doc":"Access token revocation","ref":"Boruta.Oauth.Revoke.html","title":"Boruta.Oauth.Revoke","type":"module"},{"doc":"Revokes token according to the given `Boruta.Oauth.RevokeRequest`","ref":"Boruta.Oauth.Revoke.html#token/1","title":"Boruta.Oauth.Revoke.token/1","type":"function"},{"doc":"iex> token(%RevokeRequest{\n      client_id: \"client_id\",\n      client_secret: \"client_secret\",\n      token: \"token\"\n    })\n    :ok","ref":"Boruta.Oauth.Revoke.html#token/1-examples","title":"Examples - Boruta.Oauth.Revoke.token/1","type":"function"},{"doc":"Access token context","ref":"Boruta.Oauth.AccessTokens.html","title":"Boruta.Oauth.AccessTokens","type":"behaviour"},{"doc":"Persists a token with the given params.","ref":"Boruta.Oauth.AccessTokens.html#c:create/2","title":"Boruta.Oauth.AccessTokens.create/2","type":"callback"},{"doc":"Returns a `Boruta.Oauth.Token` either by `value` or `refresh_token`.","ref":"Boruta.Oauth.AccessTokens.html#c:get_by/1","title":"Boruta.Oauth.AccessTokens.get_by/1","type":"callback"},{"doc":"Revokes the given `Boruta.Oauth.Token`.","ref":"Boruta.Oauth.AccessTokens.html#c:revoke/1","title":"Boruta.Oauth.AccessTokens.revoke/1","type":"callback"},{"doc":"Revokes only the given `Boruta.Oauth.Token` refresh token.","ref":"Boruta.Oauth.AccessTokens.html#c:revoke_refresh_token/1","title":"Boruta.Oauth.AccessTokens.revoke_refresh_token/1","type":"callback"},{"doc":"Client context","ref":"Boruta.Oauth.Clients.html","title":"Boruta.Oauth.Clients","type":"behaviour"},{"doc":"Returns client authorized scopes. The scopes will be granted for every requests to the given client.","ref":"Boruta.Oauth.Clients.html#c:authorized_scopes/1","title":"Boruta.Oauth.Clients.authorized_scopes/1","type":"callback"},{"doc":"Returns a `Boruta.Oauth.Client` given id.","ref":"Boruta.Oauth.Clients.html#c:get_client/1","title":"Boruta.Oauth.Clients.get_client/1","type":"callback"},{"doc":"Returns all clients jwk public keys","ref":"Boruta.Oauth.Clients.html#c:list_clients_jwk/0","title":"Boruta.Oauth.Clients.list_clients_jwk/0","type":"callback"},{"doc":"Code context","ref":"Boruta.Oauth.Codes.html","title":"Boruta.Oauth.Codes","type":"behaviour"},{"doc":"Persists a token according to given params.","ref":"Boruta.Oauth.Codes.html#c:create/1","title":"Boruta.Oauth.Codes.create/1","type":"callback"},{"doc":"Returns a `Boruta.Oauth.Token` by `value` and `redirect_uri`.","ref":"Boruta.Oauth.Codes.html#c:get_by/1","title":"Boruta.Oauth.Codes.get_by/1","type":"callback"},{"doc":"Revokes the given `Boruta.Oauth.Token`.","ref":"Boruta.Oauth.Codes.html#c:revoke/1","title":"Boruta.Oauth.Codes.revoke/1","type":"callback"},{"doc":"Resource owner context","ref":"Boruta.Oauth.ResourceOwners.html","title":"Boruta.Oauth.ResourceOwners","type":"behaviour"},{"doc":"Returns a list of authorized scopes for a given resource owner. These scopes will be granted is requested for the user.","ref":"Boruta.Oauth.ResourceOwners.html#c:authorized_scopes/1","title":"Boruta.Oauth.ResourceOwners.authorized_scopes/1","type":"callback"},{"doc":"Determines if given password is valid for the given resource owner.","ref":"Boruta.Oauth.ResourceOwners.html#c:check_password/2","title":"Boruta.Oauth.ResourceOwners.check_password/2","type":"callback"},{"doc":"Returns `id_token` identity claims for the given resource owner. Those claims will be present in resulting `id_token` of OpenID Connect flows.","ref":"Boruta.Oauth.ResourceOwners.html#c:claims/2","title":"Boruta.Oauth.ResourceOwners.claims/2","type":"callback"},{"doc":"Returns a resource owner by (username) or (id).","ref":"Boruta.Oauth.ResourceOwners.html#c:get_by/1","title":"Boruta.Oauth.ResourceOwners.get_by/1","type":"callback"},{"doc":"Scope context","ref":"Boruta.Oauth.Scopes.html","title":"Boruta.Oauth.Scopes","type":"behaviour"},{"doc":"","ref":"Boruta.Oauth.Scopes.html#c:public/0","title":"Boruta.Oauth.Scopes.public/0","type":"callback"},{"doc":"OAuth client schema and utilities","ref":"Boruta.Oauth.Client.html","title":"Boruta.Oauth.Client","type":"module"},{"doc":"","ref":"Boruta.Oauth.Client.html#check_redirect_uri/2","title":"Boruta.Oauth.Client.check_redirect_uri/2","type":"function"},{"doc":"","ref":"Boruta.Oauth.Client.html#check_secret/2","title":"Boruta.Oauth.Client.check_secret/2","type":"function"},{"doc":"","ref":"Boruta.Oauth.Client.html#grant_type_supported?/2","title":"Boruta.Oauth.Client.grant_type_supported?/2","type":"function"},{"doc":"Returns grant types supported by the server. `Boruta.Oauth.Client` `supported_grant_types` attribute may be a subset of them.","ref":"Boruta.Oauth.Client.html#grant_types/0","title":"Boruta.Oauth.Client.grant_types/0","type":"function"},{"doc":"","ref":"Boruta.Oauth.Client.html#public_refresh_token?/1","title":"Boruta.Oauth.Client.public_refresh_token?/1","type":"function"},{"doc":"","ref":"Boruta.Oauth.Client.html#public_revoke?/1","title":"Boruta.Oauth.Client.public_revoke?/1","type":"function"},{"doc":"","ref":"Boruta.Oauth.Client.html#should_check_secret?/2","title":"Boruta.Oauth.Client.should_check_secret?/2","type":"function"},{"doc":"","ref":"Boruta.Oauth.Client.html#t:t/0","title":"Boruta.Oauth.Client.t/0","type":"type"},{"doc":"OpenID Connect id token schema and utilities","ref":"Boruta.Oauth.IdToken.html","title":"Boruta.Oauth.IdToken","type":"module"},{"doc":"","ref":"Boruta.Oauth.IdToken.html#generate/2","title":"Boruta.Oauth.IdToken.generate/2","type":"function"},{"doc":"","ref":"Boruta.Oauth.IdToken.html#t:claims/0","title":"Boruta.Oauth.IdToken.claims/0","type":"type"},{"doc":"","ref":"Boruta.Oauth.IdToken.html#t:tokens/0","title":"Boruta.Oauth.IdToken.tokens/0","type":"type"},{"doc":"Oauth resource owner schema","ref":"Boruta.Oauth.ResourceOwner.html","title":"Boruta.Oauth.ResourceOwner","type":"module"},{"doc":"","ref":"Boruta.Oauth.ResourceOwner.html#t:t/0","title":"Boruta.Oauth.ResourceOwner.t/0","type":"type"},{"doc":"OAuth scope schema and utilities","ref":"Boruta.Oauth.Scope.html","title":"Boruta.Oauth.Scope","type":"module"},{"doc":"Determines if artifact is authorized to access given scope.","ref":"Boruta.Oauth.Scope.html#authorized_scopes/3","title":"Boruta.Oauth.Scope.authorized_scopes/3","type":"function"},{"doc":"Determines if scope string contains openid scope.","ref":"Boruta.Oauth.Scope.html#contains_openid?/1","title":"Boruta.Oauth.Scope.contains_openid?/1","type":"function"},{"doc":"Returns 'openid' scope","ref":"Boruta.Oauth.Scope.html#openid/0","title":"Boruta.Oauth.Scope.openid/0","type":"function"},{"doc":"Splits an OAuth scope string into individual scopes as string","ref":"Boruta.Oauth.Scope.html#split/1","title":"Boruta.Oauth.Scope.split/1","type":"function"},{"doc":"iex> scope(\"a:scope another:scope\")\n    [\"a:scope\", \"another:scope\"]","ref":"Boruta.Oauth.Scope.html#split/1-examples","title":"Examples - Boruta.Oauth.Scope.split/1","type":"function"},{"doc":"","ref":"Boruta.Oauth.Scope.html#t:raw/0","title":"Boruta.Oauth.Scope.raw/0","type":"type"},{"doc":"","ref":"Boruta.Oauth.Scope.html#t:t/0","title":"Boruta.Oauth.Scope.t/0","type":"type"},{"doc":"OAuth access token and code schema and utilities","ref":"Boruta.Oauth.Token.html","title":"Boruta.Oauth.Token","type":"module"},{"doc":"Determines if a token is valid, neither expired nor revoked.","ref":"Boruta.Oauth.Token.html#ensure_valid/2","title":"Boruta.Oauth.Token.ensure_valid/2","type":"function"},{"doc":"iex> ensure_valid(%Boruta.Oauth.Token{revoked_at: nil})\n    :ok\n\n    iex> ensure_valid(%Boruta.Oauth.Token{})\n    {:error, \"Token revoked.\"}","ref":"Boruta.Oauth.Token.html#ensure_valid/2-examples","title":"Examples - Boruta.Oauth.Token.ensure_valid/2","type":"function"},{"doc":"Determines if a token is expired","ref":"Boruta.Oauth.Token.html#expired?/3","title":"Boruta.Oauth.Token.expired?/3","type":"function"},{"doc":"iex> expired?(%Boruta.Oauth.Token{expires_at: 1628260754})\n    false\n\n    iex> expired?(%Boruta.Oauth.Token{expires_at: 0}) # 1st january 1970\n    true","ref":"Boruta.Oauth.Token.html#expired?/3-examples","title":"Examples - Boruta.Oauth.Token.expired?/3","type":"function"},{"doc":"Returns an hexadecimal SHA512 hash of given string","ref":"Boruta.Oauth.Token.html#hash/1","title":"Boruta.Oauth.Token.hash/1","type":"function"},{"doc":"iex> hash(\"foo\")\n    \"F7FBBA6E0636F890E56FBBF3283E524C6FA3204AE298382D624741D0DC6638326E282C41BE5E4254D8820772C5518A2C5A8C0C7F7EDA19594A7EB539453E1ED7\"","ref":"Boruta.Oauth.Token.html#hash/1-examples","title":"Examples - Boruta.Oauth.Token.hash/1","type":"function"},{"doc":"Determines if a token is revoked.","ref":"Boruta.Oauth.Token.html#revoked?/2","title":"Boruta.Oauth.Token.revoked?/2","type":"function"},{"doc":"iex> revoked?(%Boruta.Oauth.Token{revoked_at: nil})\n    :ok\n\n    iex> revoked?(%Boruta.Oauth.Token{})\n    false","ref":"Boruta.Oauth.Token.html#revoked?/2-examples","title":"Examples - Boruta.Oauth.Token.revoked?/2","type":"function"},{"doc":"","ref":"Boruta.Oauth.Token.html#userinfo/1","title":"Boruta.Oauth.Token.userinfo/1","type":"function"},{"doc":"","ref":"Boruta.Oauth.Token.html#t:t/0","title":"Boruta.Oauth.Token.t/0","type":"type"},{"doc":"Authorization code request","ref":"Boruta.Oauth.AuthorizationCodeRequest.html","title":"Boruta.Oauth.AuthorizationCodeRequest","type":"module"},{"doc":"Type representing an authorization code request as stated in [OAuth 2.0 RFC](https://tools.ietf.org/html/rfc6749#section-4.1.3).","ref":"Boruta.Oauth.AuthorizationCodeRequest.html#t:t/0","title":"Boruta.Oauth.AuthorizationCodeRequest.t/0","type":"type"},{"doc":"Client credentials request","ref":"Boruta.Oauth.ClientCredentialsRequest.html","title":"Boruta.Oauth.ClientCredentialsRequest","type":"module"},{"doc":"Type representing a client credentials request as stated in [OAuth 2.0 RFC](https://tools.ietf.org/html/rfc6749#section-4.4.2).","ref":"Boruta.Oauth.ClientCredentialsRequest.html#t:t/0","title":"Boruta.Oauth.ClientCredentialsRequest.t/0","type":"type"},{"doc":"Code request","ref":"Boruta.Oauth.CodeRequest.html","title":"Boruta.Oauth.CodeRequest","type":"module"},{"doc":"","ref":"Boruta.Oauth.CodeRequest.html#require_nonce?/1","title":"Boruta.Oauth.CodeRequest.require_nonce?/1","type":"function"},{"doc":"Type representing a code request as stated in [OAuth 2.0 RFC](https://tools.ietf.org/html/rfc6749#section-4.1.1).\n\nNote : `resource_owner` is an addition that must be provided by the application layer.","ref":"Boruta.Oauth.CodeRequest.html#t:t/0","title":"Boruta.Oauth.CodeRequest.t/0","type":"type"},{"doc":"Hybrid request","ref":"Boruta.Oauth.HybridRequest.html","title":"Boruta.Oauth.HybridRequest","type":"module"},{"doc":"Type representing an hybrid request as stated in [OpenId Connect core 1.0](https://openid.net/specs/openid-connect-core-1_0.html#HybridFlowAuth).\n\nNote : `resource_owner` is an addition that must be provided by the application layer.","ref":"Boruta.Oauth.HybridRequest.html#t:t/0","title":"Boruta.Oauth.HybridRequest.t/0","type":"type"},{"doc":"Introspect request","ref":"Boruta.Oauth.IntrospectRequest.html","title":"Boruta.Oauth.IntrospectRequest","type":"module"},{"doc":"Type representing an introspect request as stated in [Introspect RFC](https://tools.ietf.org/html/rfc7662#section-2.1).","ref":"Boruta.Oauth.IntrospectRequest.html#t:t/0","title":"Boruta.Oauth.IntrospectRequest.t/0","type":"type"},{"doc":"Resource owner password credentials request","ref":"Boruta.Oauth.PasswordRequest.html","title":"Boruta.Oauth.PasswordRequest","type":"module"},{"doc":"Type representing a resource owner password credentials request as stated in [OAuth 2.0 RFC](https://tools.ietf.org/html/rfc6749#section-4.3.2).","ref":"Boruta.Oauth.PasswordRequest.html#t:t/0","title":"Boruta.Oauth.PasswordRequest.t/0","type":"type"},{"doc":"Refresh token request","ref":"Boruta.Oauth.RefreshTokenRequest.html","title":"Boruta.Oauth.RefreshTokenRequest","type":"module"},{"doc":"Type representing a refresh token request as stated in [OAuth 2.0 RFC](https://tools.ietf.org/html/rfc6749#section-1.5).","ref":"Boruta.Oauth.RefreshTokenRequest.html#t:t/0","title":"Boruta.Oauth.RefreshTokenRequest.t/0","type":"type"},{"doc":"Build an oauth request struct from given input.\n\n> __Note__: Input must have the shape or be a `%Plug.Conn{}` request.","ref":"Boruta.Oauth.Request.html","title":"Boruta.Oauth.Request","type":"module"},{"doc":"Create request struct from an OAuth authorize request.\n\nNote : resource owner must be provided as current_user assigns.","ref":"Boruta.Oauth.Request.html#authorize_request/2","title":"Boruta.Oauth.Request.authorize_request/2","type":"function"},{"doc":"iex>authorize_request(\n      %{\n        query_params: %{\n          \"response_type\" => \"token\",\n          \"client_id\" => \"client_id\",\n          \"redirect_uri\" => \"redirect_uri\"\n        },\n      },\n      %User{...}\n    )\n    {:ok, %TokenRequest{...}}","ref":"Boruta.Oauth.Request.html#authorize_request/2-examples","title":"Examples - Boruta.Oauth.Request.authorize_request/2","type":"function"},{"doc":"Create request struct from an OAuth introspect request.","ref":"Boruta.Oauth.Request.html#introspect_request/1","title":"Boruta.Oauth.Request.introspect_request/1","type":"function"},{"doc":"iex>introspect_request(%{\n      body_params: %{\n        \"token\" => \"token\",\n        \"client_id\" => \"client_id\",\n        \"client_secret\" => \"client_secret\",\n      }\n    })\n    {:ok, %IntrospectRequest{...}}","ref":"Boruta.Oauth.Request.html#introspect_request/1-examples","title":"Examples - Boruta.Oauth.Request.introspect_request/1","type":"function"},{"doc":"Create request struct from an OAuth revoke request.","ref":"Boruta.Oauth.Request.html#revoke_request/1","title":"Boruta.Oauth.Request.revoke_request/1","type":"function"},{"doc":"iex>revoke_request(%{\n      body_params: %{\n        \"token\" => \"token\",\n        \"client_id\" => \"client_id\",\n        \"client_secret\" => \"client_secret\",\n      }\n    })\n    {:ok, %RevokeRequest{...}}","ref":"Boruta.Oauth.Request.html#revoke_request/1-examples","title":"Examples - Boruta.Oauth.Request.revoke_request/1","type":"function"},{"doc":"Create request struct from an OAuth token request.","ref":"Boruta.Oauth.Request.html#token_request/1","title":"Boruta.Oauth.Request.token_request/1","type":"function"},{"doc":"iex>token_request(%{\n      body_params: %{\n        \"grant_type\" => \"client_credentials\",\n        \"client_id\" => \"client_id\",\n        \"client_secret\" => \"client_secret\"\n      }\n    })\n    {:ok, %ClientCredentialsRequest{...}}","ref":"Boruta.Oauth.Request.html#token_request/1-examples","title":"Examples - Boruta.Oauth.Request.token_request/1","type":"function"},{"doc":"Revoke request","ref":"Boruta.Oauth.RevokeRequest.html","title":"Boruta.Oauth.RevokeRequest","type":"module"},{"doc":"Type representing an revoke request as stated in [OAuth 2.0 Token revocation RFC](https://tools.ietf.org/html/rfc7009#section-2.1).","ref":"Boruta.Oauth.RevokeRequest.html#t:t/0","title":"Boruta.Oauth.RevokeRequest.t/0","type":"type"},{"doc":"Implicit request","ref":"Boruta.Oauth.TokenRequest.html","title":"Boruta.Oauth.TokenRequest","type":"module"},{"doc":"","ref":"Boruta.Oauth.TokenRequest.html#require_nonce?/1","title":"Boruta.Oauth.TokenRequest.require_nonce?/1","type":"function"},{"doc":"Type representing an implicit request as stated in [OAuth 2.0 RFC](https://tools.ietf.org/html/rfc6749#section-4.2.1) and [OpenId Connect core 1.0](https://openid.net/specs/openid-connect-core-1_0.html#ImplicitFlowAuth)\n\nNote : `resource_owner` is an addition that must be provided by the application layer.","ref":"Boruta.Oauth.TokenRequest.html#t:t/0","title":"Boruta.Oauth.TokenRequest.t/0","type":"type"},{"doc":"Encapsulate injected `Boruta.Oauth.AccessTokens` adapter in context configuration","ref":"Boruta.AccessTokensAdapter.html","title":"Boruta.AccessTokensAdapter","type":"module"},{"doc":"","ref":"Boruta.AccessTokensAdapter.html#create/2","title":"Boruta.AccessTokensAdapter.create/2","type":"function"},{"doc":"","ref":"Boruta.AccessTokensAdapter.html#get_by/1","title":"Boruta.AccessTokensAdapter.get_by/1","type":"function"},{"doc":"","ref":"Boruta.AccessTokensAdapter.html#revoke/1","title":"Boruta.AccessTokensAdapter.revoke/1","type":"function"},{"doc":"","ref":"Boruta.AccessTokensAdapter.html#revoke_refresh_token/1","title":"Boruta.AccessTokensAdapter.revoke_refresh_token/1","type":"function"},{"doc":"Boruta Ecto adapter uses [Nebulex](https://github.com/cabol/nebulex) in order to cache entities","ref":"Boruta.Cache.html","title":"Boruta.Cache","type":"module"},{"doc":"A convenience function for getting the primary storage cache.","ref":"Boruta.Cache.html#__primary__/0","title":"Boruta.Cache.__primary__/0","type":"function"},{"doc":"A convenience function for joining the cache to the cluster.","ref":"Boruta.Cache.html#join_cluster/0","title":"Boruta.Cache.join_cluster/0","type":"function"},{"doc":"A convenience function for removing the cache from the cluster.","ref":"Boruta.Cache.html#leave_cluster/0","title":"Boruta.Cache.leave_cluster/0","type":"function"},{"doc":"A convenience function for getting the cluster nodes.","ref":"Boruta.Cache.html#nodes/0","title":"Boruta.Cache.nodes/0","type":"function"},{"doc":"This is the cache for the primary storage.","ref":"Boruta.Cache.Primary.html","title":"Boruta.Cache.Primary","type":"module"},{"doc":"A convenience function for retrieving the current generations.","ref":"Boruta.Cache.Primary.html#generations/0","title":"Boruta.Cache.Primary.generations/0","type":"function"},{"doc":"A convenience function for creating new generations.","ref":"Boruta.Cache.Primary.html#new_generation/1","title":"Boruta.Cache.Primary.new_generation/1","type":"function"},{"doc":"A convenience function for retrieving the newer generation.","ref":"Boruta.Cache.Primary.html#newer_generation/0","title":"Boruta.Cache.Primary.newer_generation/0","type":"function"},{"doc":"A convenience function for reset the GC timer.","ref":"Boruta.Cache.Primary.html#reset_generation_timer/0","title":"Boruta.Cache.Primary.reset_generation_timer/0","type":"function"},{"doc":"Encapsulate injected `Boruta.Oauth.Clients` adapter in context configuration","ref":"Boruta.ClientsAdapter.html","title":"Boruta.ClientsAdapter","type":"module"},{"doc":"","ref":"Boruta.ClientsAdapter.html#authorized_scopes/1","title":"Boruta.ClientsAdapter.authorized_scopes/1","type":"function"},{"doc":"","ref":"Boruta.ClientsAdapter.html#create_client/1","title":"Boruta.ClientsAdapter.create_client/1","type":"function"},{"doc":"","ref":"Boruta.ClientsAdapter.html#get_client/1","title":"Boruta.ClientsAdapter.get_client/1","type":"function"},{"doc":"","ref":"Boruta.ClientsAdapter.html#list_clients_jwk/0","title":"Boruta.ClientsAdapter.list_clients_jwk/0","type":"function"},{"doc":"","ref":"Boruta.ClientsAdapter.html#refresh_jwk_from_jwks_uri/1","title":"Boruta.ClientsAdapter.refresh_jwk_from_jwks_uri/1","type":"function"},{"doc":"Encapsulate injected `Boruta.Oauth.Codes` adapter in context configuration","ref":"Boruta.CodesAdapter.html","title":"Boruta.CodesAdapter","type":"module"},{"doc":"","ref":"Boruta.CodesAdapter.html#create/1","title":"Boruta.CodesAdapter.create/1","type":"function"},{"doc":"","ref":"Boruta.CodesAdapter.html#get_by/1","title":"Boruta.CodesAdapter.get_by/1","type":"function"},{"doc":"","ref":"Boruta.CodesAdapter.html#revoke/1","title":"Boruta.CodesAdapter.revoke/1","type":"function"},{"doc":"Encapsulate injected `Boruta.Oauth.Scopes` adapter in context configuration","ref":"Boruta.ScopesAdapter.html","title":"Boruta.ScopesAdapter","type":"module"},{"doc":"","ref":"Boruta.ScopesAdapter.html#public/0","title":"Boruta.ScopesAdapter.public/0","type":"function"},{"doc":"Ecto Adapter Client Schema","ref":"Boruta.Ecto.Client.html","title":"Boruta.Ecto.Client","type":"module"},{"doc":"","ref":"Boruta.Ecto.Client.html#create_changeset/2","title":"Boruta.Ecto.Client.create_changeset/2","type":"function"},{"doc":"","ref":"Boruta.Ecto.Client.html#key_pair_changeset/2","title":"Boruta.Ecto.Client.key_pair_changeset/2","type":"function"},{"doc":"","ref":"Boruta.Ecto.Client.html#secret_changeset/2","title":"Boruta.Ecto.Client.secret_changeset/2","type":"function"},{"doc":"","ref":"Boruta.Ecto.Client.html#update_changeset/2","title":"Boruta.Ecto.Client.update_changeset/2","type":"function"},{"doc":"","ref":"Boruta.Ecto.Client.html#t:t/0","title":"Boruta.Ecto.Client.t/0","type":"type"},{"doc":"Ecto Adapter Scope Schema","ref":"Boruta.Ecto.Scope.html","title":"Boruta.Ecto.Scope","type":"module"},{"doc":"","ref":"Boruta.Ecto.Scope.html#assoc_changeset/2","title":"Boruta.Ecto.Scope.assoc_changeset/2","type":"function"},{"doc":"","ref":"Boruta.Ecto.Scope.html#changeset/2","title":"Boruta.Ecto.Scope.changeset/2","type":"function"},{"doc":"","ref":"Boruta.Ecto.Scope.html#t:t/0","title":"Boruta.Ecto.Scope.t/0","type":"type"},{"doc":"Ecto Adapter Token Schema","ref":"Boruta.Ecto.Token.html","title":"Boruta.Ecto.Token","type":"module"},{"doc":"","ref":"Boruta.Ecto.Token.html#changeset/2","title":"Boruta.Ecto.Token.changeset/2","type":"function"},{"doc":"","ref":"Boruta.Ecto.Token.html#changeset_with_refresh_token/2","title":"Boruta.Ecto.Token.changeset_with_refresh_token/2","type":"function"},{"doc":"","ref":"Boruta.Ecto.Token.html#code_changeset/2","title":"Boruta.Ecto.Token.code_changeset/2","type":"function"},{"doc":"","ref":"Boruta.Ecto.Token.html#t:t/0","title":"Boruta.Ecto.Token.t/0","type":"type"},{"doc":"HTTP BasicAuth utilities\n\nProvide utilities to decode Basic authorization header as stated in [RFC 7617 - The 'Basic' HTTP Authentication Scheme](https://tools.ietf.org/html/rfc7617)","ref":"Boruta.BasicAuth.html","title":"Boruta.BasicAuth","type":"module"},{"doc":"Decode given authorization header and returns an array containing username and password.","ref":"Boruta.BasicAuth.html#decode/1","title":"Boruta.BasicAuth.decode/1","type":"function"},{"doc":"iex> Boruta.BasicAuth.decode(\"Basic dXNlcm5hbWU6cGFzc3dvcmQ=\")\n    {:ok, [\"username\", \"password\"]}\n\n    iex> Boruta.BasicAuth.decode(\"bad_authorization_header\")\n    {:error, \"`bad_authorization_header` is not a valid Basic authorization header.\"}","ref":"Boruta.BasicAuth.html#decode/1-examples","title":"Examples - Boruta.BasicAuth.decode/1","type":"function"},{"doc":"OAuth bearer token utilities\n\nProvide utilities to manipulate bearer tokens as stated in [RFC 6750 - Bearer token usage](https://datatracker.ietf.org/doc/html/rfc6750)","ref":"Boruta.Oauth.BearerToken.html","title":"Boruta.Oauth.BearerToken","type":"module"},{"doc":"","ref":"Boruta.Oauth.BearerToken.html#extract_token/1","title":"Boruta.Oauth.BearerToken.extract_token/1","type":"function"},{"doc":"Behaviour to implement utilities to generate token value. This must be implemented in the module configured as `token_generator` set in `config.exs`","ref":"Boruta.Oauth.TokenGenerator.html","title":"Boruta.Oauth.TokenGenerator","type":"behaviour"},{"doc":"Generates a token value from token entity.","ref":"Boruta.Oauth.TokenGenerator.html#c:generate/2","title":"Boruta.Oauth.TokenGenerator.generate/2","type":"callback"},{"doc":"Generates a secret from client entity.","ref":"Boruta.Oauth.TokenGenerator.html#c:secret/1","title":"Boruta.Oauth.TokenGenerator.secret/1","type":"callback"},{"doc":"Utility to validate the request according to the given parameters","ref":"Boruta.Oauth.Validator.html","title":"Boruta.Oauth.Validator","type":"module"},{"doc":"Validates given OAuth parameters.","ref":"Boruta.Oauth.Validator.html#validate/2","title":"Boruta.Oauth.Validator.validate/2","type":"function"},{"doc":"iex> validate(:token, %{\n      \"grant_type\" => \"client_credentials\",\n      \"client_id\" => \"client_id\",\n      \"client_secret\" => \"client_secret\"\n    })\n    {:ok, %{\n      \"grant_type\" => \"client_credentials\",\n      \"client_id\" => \"client_id\",\n      \"client_secret\" => \"client_secret\"\n    }}\n\n    iex> validate(:authorize, %{})\n    {:error, \"Request is not a valid OAuth request. Need a response_type param.\"}","ref":"Boruta.Oauth.Validator.html#validate/2-examples","title":"Examples - Boruta.Oauth.Validator.validate/2","type":"function"},{"doc":"Boruta OAuth errors\n\n> __Note__: Intended to follow [OAuth 2.0 errors](https://tools.ietf.org/html/rfc6749#section-5.2). Additional errors are provided as purpose.","ref":"Boruta.Oauth.Error.html","title":"Boruta.Oauth.Error","type":"module"},{"doc":"Returns the URL to be redirected according to error format.","ref":"Boruta.Oauth.Error.html#redirect_to_url/1","title":"Boruta.Oauth.Error.redirect_to_url/1","type":"function"},{"doc":"Returns the OAuth error augmented with the format according to request type.","ref":"Boruta.Oauth.Error.html#with_format/2","title":"Boruta.Oauth.Error.with_format/2","type":"function"},{"doc":"","ref":"Boruta.Oauth.Error.html#t:t/0","title":"Boruta.Oauth.Error.t/0","type":"type"},{"doc":"[![downloads](https://img.shields.io/hexpm/dt/boruta)](https://hex.pm/packages/boruta)\n![continuous integration](https://github.com/malach-it/boruta_auth/actions/workflows/elixir.yml/badge.svg)\n[![coverage Status](https://coveralls.io/repos/malach-it/boruta_auth/badge.svg?branch=master)](https://coveralls.io/r/malach-it/boruta_auth?branch=master)\n\n![Logo](https://github.com/malach-it/boruta_auth/raw/master/images/logo-yellow.png)\n\n# Boruta OAuth/OpenID Connect provider core\n\nBoruta is the core of an OAuth 2.0 and OpenID Connect provider implementing according business rules. This library also provides a generator to create phoenix controllers, views and templates to have a basic provider up and running.\n\nAs it, a provider implemented using Boruta aim to follow RFCs:\n- [RFC 6749 - The OAuth 2.0 Authorization Framework](https://tools.ietf.org/html/rfc6749)\n- [RFC 7662 - OAuth 2.0 Token Introspection](https://tools.ietf.org/html/rfc7662)\n- [RFC 7009 - OAuth 2.0 Token Revocation](https://tools.ietf.org/html/rfc7009)\n- [RFC 7636 - Proof Key for Code Exchange by OAuth Public Clients](https://tools.ietf.org/html/rfc7636)\n- [RFC 7521 - Assertion Framework for OAuth 2.0 Client Authentication and Authorization Grants](https://www.rfc-editor.org/rfc/rfc7521)\n- [RFC 7523 - JSON Web Token (JWT) Profile for OAuth 2.0 Client Authentication and Authorization Grants](https://tools.ietf.org/html/rfc7523)\n\nAnd specification from OpenID Foundation:\n- [OpenID Connect core 1.0](https://openid.net/specs/openid-connect-core-1_0.html)\n- [OpenID Connect Dynamic Client Registration 1.0 incorporating errata set 1](https://openid.net/specs/openid-connect-registration-1_0.html)\n\nThis package is meant to help to bring authorization into Elixir applications. With it, you can perform part or all of authorization code, implicit, hybrid, client credentials, or resource owner password credentials grants flows. It also helps introspecting and revoking tokens.","ref":"readme.html","title":"README","type":"extras"},{"doc":"Master branch documentation can be found [here](https://github.com/malach-it/boruta_auth/blob/master/README.md)\n\nStable documentation is hosted on [hexdocs.pm](https://hexdocs.pm/boruta/api-reference.html)","ref":"readme.html#documentation","title":"Documentation - README","type":"extras"},{"doc":"An example of integration can be found [here](https://github.com/patatoid/boruta_example), it followed the integration steps described in below guides section.","ref":"readme.html#integration-example","title":"Integration example - README","type":"extras"},{"doc":"This package has successfully passed basic, implicit and hybrid OpenID Profiles certifications as of May 7th, 2022 for its version [2.1.2](https://hex.pm/packages/boruta/2.1.2). This certification was performed with the above sample server.\n\n![OpenID Certification watermark](https://github.com/malach-it/boruta_auth/raw/master/images/oid-certification-mark.png)","ref":"readme.html#openid-certification","title":"OpenID Certification - README","type":"extras"},{"doc":"Here are some guides helping the integration of OAuth/OpenID Connect in your systems:\n\n- [Basic OAuth/OpenID Connect provider integration](guides/provider_integration.md)\n- [How to create an OAuth client](guides/create_client.md)\n- [Client request authorization](guides/authorize_requests.md)\n- [Notes about confidential clients](guides/confidential_clients.md)\n- [Notes about pkce](guides/pkce.md)","ref":"readme.html#guides","title":"Guides - README","type":"extras"},{"doc":"It is a work in progress, all feedbacks / feature requests / improvements are welcome","ref":"readme.html#feedback","title":"Feedback - README","type":"extras"},{"doc":"This product community follows the code of conduct available [here](https://io.malach.it/code-of-conduct.html)","ref":"readme.html#code-of-conduct","title":"Code of Conduct - README","type":"extras"},{"doc":"This code is released under the [MIT](LICENSE.md) license.","ref":"readme.html#license","title":"License - README","type":"extras"},{"doc":"# Setting up Boruta OAuth/OpenID Connect provider\nBoruta provides as its core all authorization business rules in order to handle the underlying authorization logic of OAuth and OpenID Connect. Then it provides a generator that helps to create required controllers, views, and templates as we will see.","ref":"provider_integration.html","title":"Setting up Boruta OAuth/OpenID Connect provider","type":"extras"},{"doc":"We will start by botsrapping a Phoenix web application with authentication capabilities provided by `phx.gen.auth` since OAuth and OpenID Connect specifications do not provide any recommendations about how to authenticate users. Instead, it provides all protocols required to authorize them to secure an HTTP service, with relative identity information in destination to the client brought by OpenID Connect core. Here we go, first bootstrapping the application:\n```sh\n~> mix phx.new boruta_example\n```\nThen the authentication\n```sh\n~> mix phx.gen.auth Accounts User users\n```\nWe have now a web application in which we can log in. If you want to know more about those, have a look at [phx.new](https://hexdocs.pm/phoenix/Mix.Tasks.Phx.New.html) and [phx.gen.auth](https://hexdocs.pm/phoenix/Mix.Tasks.Phx.Gen.Auth.html) documentations.\n\nIn order to run the newly created application, you have to set up dependencies and the database. You'll find the development database configuration in `config/dev.exs` file, fill it with valid PostgreSQL credentials, you would be able to run\n```sh\n~> mix do deps.get, ecto.setup\n```\nHere is the application up and running, starting the webserver with `mix phx.server` you'll be able to visit `http://localhost:4000` with your favorite browser.","ref":"provider_integration.html#1-bootstrap-the-application","title":"1. Bootstrap the application - Setting up Boruta OAuth/OpenID Connect provider","type":"extras"},{"doc":"Once the application is up, we can go on to the authorization part. First, you can add the Boruta dependency in `mix.exs`\n```elixir\n# mix.exs\n\n  def deps do\n  ...\n      {:boruta, \"~> 2.0\"}\n  ...\n  end\n```\n\nAfter that, you'll be able to generate controllers in order to expose Oauth and OpenID Connect core specifications endpoints and database migrations to persist required clients, scopes, and tokens needed by your newly created provider.\n\n```sh\n~> mix do deps.get, boruta.gen.migration, ecto.migrate, boruta.gen.controllers\n```\n\nIt will print the remaining steps to have the provider up and running as described in the [documentation](https://hexdocs.pm/boruta/Mix.Tasks.Boruta.Gen.Controllers.html). From there we will skip the testing part which uses Mox in order to mock Boruta and focus tests on the application layer.","ref":"provider_integration.html#2-bootstrap-boruta","title":"2. Bootstrap Boruta - Setting up Boruta OAuth/OpenID Connect provider","type":"extras"},{"doc":"As described in `boruta.gen.controllers` mix task output, you need to expose controller actions in `router.ex` as follow\n\n```elixir\n# lib/boruta_example_web/router.ex\n\n  scope \"/oauth\", BorutaExampleWeb.Oauth do\n    pipe_through :api\n\n    post \"/revoke\", RevokeController, :revoke\n    post \"/token\", TokenController, :token\n    post \"/introspect\", IntrospectController, :introspect\n  end\n\n  scope \"/oauth\", BorutaExampleWeb.Oauth do\n    pipe_through [:browser, :fetch_current_user]\n\n    get \"/authorize\", AuthorizeController, :authorize\n  end\n\n  scope \"/openid\", BorutaExampleWeb.Openid do\n    pipe_through [:browser, :fetch_current_user]\n\n    get \"/authorize\", AuthorizeController, :authorize\n  end\n```\n\nAnd give mandatory boruta configuration\n```elixir\n# config/config.exs\n\nconfig :boruta, Boruta.Oauth,\n  repo: BorutaExample.Repo,\n  issuer: \"https://example.com\"\n```\nHere client credentials flow should be up. For user flows you need further configuration and implement `Boruta.Oauth.ResourceOwners` context.","ref":"provider_integration.html#3-configure-boruta","title":"3. Configure Boruta - Setting up Boruta OAuth/OpenID Connect provider","type":"extras"},{"doc":"In order to have user flows operational, you need to implement `Boruta.Oauth.ResourceOwners` context as described in [Boruta README](https://github.com/malach-it/boruta_auth/blob/master/README.md). Here it would look like\n```elixir\n# lib/boruta_example/resource_owners.ex\n\ndefmodule BorutaExample.ResourceOwners do\n  @behaviour Boruta.Oauth.ResourceOwners\n\n  alias Boruta.Oauth.ResourceOwner\n  alias MyApp.Accounts.User\n  alias MyApp.Repo\n\n  @impl Boruta.Oauth.ResourceOwners\n  def get_by(username: username) do\n    with %User{id: id, email: email, last_login_at: last_login_at}   {:error, \"User not found.\"}\n    end\n  end\n  def get_by(sub: sub) do\n    with %User{id: id, email: email, last_login_at: last_login_at}   {:error, \"User not found.\"}\n    end\n  end\n\n  @impl Boruta.Oauth.ResourceOwners\n  def check_password(resource_owner, password) do\n    user = Repo.get_by(User, id: resource_owner.sub)\n    case User.valid_password?(user, password) do\n      true -> :ok\n      false -> {:error, \"Invalid email or password.\"}\n    end\n  end\n\n  @impl Boruta.Oauth.ResourceOwners\n  def authorized_scopes(%ResourceOwner{}), do: []\n\n\n  @impl Boruta.Oauth.ResourceOwners\n  def claims(_resource_owner, _scope), do: %{}\nend\n```\n\nand inject it with the main configuration\n\n```elixir\n# config/config.exs\n\nconfig :boruta, Boruta.Oauth,\n  repo: BorutaExample.Repo,\n  contexts: [\n    resource_owners: BorutaExample.ResourceOwners\n  ]\n```\n\nLast, you'll have to setup is the redirection in the OAuth authorize controller\n\n```elixir\n# lib/boruta_example_web/controllers/oauth/authorize_controller.ex\n\n...\n  defp redirect_to_login(conn) do\n    redirect(conn, to: Routes.user_session_path(conn, :new))\n  end\n```\n\nHere all OAuth flows should be up and running!","ref":"provider_integration.html#4-user-flows","title":"4. User flows - Setting up Boruta OAuth/OpenID Connect provider","type":"extras"},{"doc":"In order to setup OpenID Connect flows, you need to tweak `phx.gen.auth` in order to redirect to login after logging out\n```elixir\n# lib/boruta_example_web/controllers/user_auth.ex:80\n\n...\n  def log_out_user(conn) do\n    ...\n    conn\n    |> renew_session()\n    |> delete_resp_cookie(@remember_me_cookie)\n    |> redirect(to: Routes.user_session_path(conn, :new))\n  end\n```\n\nAnd set redirections in the OpenID authorize controller\n```elixir\n# lib/boruta_example_web/controllers/openid/authorize_controller.ex\n\n...\n  defp redirect_to_login(conn) do\n    redirect(conn, to: Routes.user_session_path(conn, :new))\n  end\n\n  defp log_out_user(conn) do\n    UserAuth.log_out_user(conn)\n  end\n```\n\nLast, you need to store the `last_login_at` field of users to keep track of the timestamp when users log in\n\n```elixir\n# priv/repo/migrations/ _add_last_login_at_to_users.exs\n\ndefmodule BorutaExample.Repo.Migrations.AddLastLoginAtToUsers do\n  use Ecto.Migration\n\n  def change do\n    alter table(:users) do\n      add :last_login_at, :utc_datetime_usec\n    end\n  end\nend\n\n# lib/boruta_example/accounts.ex:223\n\n...\n  def update_last_login_at(user) do\n    user |> User.login_changeset() |> Repo.update!()\n  end\n\n...\n  def log_in_user(conn, user, params \\\\ %{}) do\n    token = Accounts.generate_user_session_token(user)\n    Accounts.update_last_login_at(user)\n    ...\n\n# lib/boruta_example/accounts/user.ex\n\n...\n  schema \"users\" do\n    ...\n    field :last_login_at, :utc_datetime_usec\n\n    timestamps()\n  end\n...\n   def login_changeset(user) do\n     change(user, last_login_at: DateTime.utc_now())\n   end\n```\n\nHere we are! You have a basic OpenID Connect provider. You can now create a client as described [here](https://hexdocs.pm/boruta/create_client.html) and start using it.\nDeparting from there, you can use any OAuth/OpenID Connect client of your choice.\n\nHope you enjoyed the way so far. The process can definitely be improved at some points, all contributions of any kind will be very welcome.","ref":"provider_integration.html#5-openid-connect","title":"5. OpenID Connect - Setting up Boruta OAuth/OpenID Connect provider","type":"extras"},{"doc":"# How to create an OAuth client\n\nBoruta goes with some __administration utilities__. The best way to have an OAuth client up and running is to __create a client with a seed script__ as following example:\n\n```elixir\n# priv/repo/seeds.exs\n\nid = SecureRandom.uuid()\nsecret = SecureRandom.hex(64)\n\nBoruta.Ecto.Admin.create_client(%{\n  id: id, # OAuth client_id\n  secret: secret, # OAuth client_secret\n  name: \"A client\", # Display name\n  access_token_ttl: 60 * 60 * 24, # one day\n  authorization_code_ttl: 60, # one minute\n  refresh_token_ttl: 60 * 60 * 24 * 30, # one month\n  id_token_ttl: 60 * 60 * 24, # one day\n  id_token_signature_alg: \"RS256\", # ID token signature algorithm, defaults to \"RS512\"\n  userinfo_signed_response_alg: \"RS256\", # userinfo signature algorithm, defaults to nil (no signature)\n  redirect_uris: [\"http://redirect.uri\"], # OAuth client redirect_uris\n  authorize_scope: true, # take following authorized_scopes into account (skip public scopes)\n  authorized_scopes: [%{name: \"a:scope\"}], # scopes that are authorized using this client\n  supported_grant_types: [ # client supported grant types\n    \"client_credentials\",\n    \"password\",\n    \"authorization_code\",\n    \"refresh_token\",\n    \"implicit\",\n    \"revoke\",\n    \"introspect\"\n  ],\n  pkce: false, # PKCE enabled\n  public_refresh_token: false, # do not require client_secret for refreshing tokens\n  public_revoke: false, # do not require client_secret for revoking tokens\n  confidential: true, # see OAuth 2.0 confidentiality (requires client secret for some flows)\n  token_endpont_auth_methods: [ # activable client authentication methods\n    \"client_secret_basic\",\n    \"client_secret_post\",\n    \"client_secret_jwt\",\n    \"private_key_jwt\"\n  ],\n  token_endpoint_jwt_auth_alg: nil, # associated to authentication methods, the algorithm to use along\n  jwt_public_key: nil # pem public key to be used with `private_key_jwt` authentication method\n}) |> IO.inspect\n```\n\nOr so, you can use all administration utilities described in [Boruta API documentation](https://hexdocs.pm/boruta/Boruta.Ecto.Admin.html) to manage all entities you need to have your server up and running. If some are missing or can be improved do not hesitate to open an issue on [GitLab](https://gitlab.com/patatoid/boruta_auth/-/issues), it would be very welcome.","ref":"create_client.html","title":"How to create an OAuth client","type":"extras"},{"doc":"# Client request authorization\n\nOnce your authorization server setup done, you can deliver tokens that help __limiting access to HTTP services__. In order to do so, you can __check validity and security information of access tokens__ provided in requests. Here we will see how restrict access using a bearer token as described in [RFC](https://datatracker.ietf.org/doc/html/rfc6750).","ref":"authorize_requests.html","title":"Client request authorization","type":"extras"},{"doc":"In a monolith, you have access to __Boruta API__ (documented [here](https://hexdocs.pm/boruta/api-reference.html)) and can directly use it in order to restrict access to endpoints. Creating a __Plug__ and add it to the __request pipeline__ would be the preferred way to perform authorization. Here is an example of basic plugs.\n\n```elixir\ndef MyAppWeb.Plugs.Authorization do\n  import Plug\n\n  use MyAppWeb, :controller\n\n  alias Boruta.Oauth.Authorization\n  alias Boruta.Oauth.Scope\n\n  def require_authenticated(conn, _opts) do\n    with [authorization_header]   assign(:current_token, token)\n      |> assign(:current_user, Accounts.get_user!(token.sub))\n    else\n      _ ->\n        conn\n        |> put_status(:unauthorized)\n        |> put_view(MyAppWeb.ErrorView)\n        |> render(\"401.json\")\n        |> halt()\n    end\n  end\n\n  def authorize(conn, [_h | _t] = required_scopes) do\n    current_scopes = Scope.split(conn.assigns[:current_token].scope)\n\n    case Enum.empty?(required_scopes -- current_scopes) do\n      true ->\n        conn\n\n      false ->\n        conn\n        |> put_status(:forbidden)\n        |> put_view(MyAppWeb.ErrorView)\n        |> render(\"403.json\")\n        |> halt()\n    end\n  end\nend\n```\n\nThen you can invoke those plugs in your router and controllers:\n```elixir\n# lib/my_app_web/router.ex\n...\n  import MyAppWeb.Plugs.Authorization,\n    only: [\n      require_authenticated: 2\n    ]\n\n  pipeline :protected_api do\n    plug(:accepts, [\"json\"])\n\n    plug(:require_authenticated)\n  end\n...\n\n# in controllers\n...\n  import MyAppWeb.Plugs.Authorization,\n    only: [\n      authorize: 2\n    ]\n\n  plug(:authorize, [\"resource:read\"]) when action in [:index, :show]\n  plug(:authorize, [\"resource:write\"]) when action in [:create, :update, :delete]\n...\n```","ref":"authorize_requests.html#in-a-monolithic-application","title":"In a monolithic application - Client request authorization","type":"extras"},{"doc":"With an authorization server set up, an __introspect endpoint__ is exposed to check token validity and provide security information as described in [RFC](https://datatracker.ietf.org/doc/html/rfc7662.html). You can create your own plugs as above but instead of using Boruta API, __request the authorization server__ to get an introspected token and all information needed to perform authorization.\n\n> Note: the grant type `introspect` must be active on the client you are performing the requests with.\n\nExample of introspect response:\n```json\n{\n    \"active\": true,\n    \"client_id\": \"6a2f41a3-c54c-fce8-32d2-0324e1c32e20\",\n    \"exp\": 1639752235,\n    \"iat\": 1639748635,\n    \"iss\": \"https://oauth.boruta.patatoid.fr\",\n    \"scope\": \"resource:read resource:write\",\n    \"sub\": \"b69c4bb6-a47b-4254-9d87-cf42bb223262\",\n    \"username\": \"test@test.test\"\n}\n```","ref":"authorize_requests.html#in-a-microservice-environment","title":"In a microservice environment - Client request authorization","type":"extras"},{"doc":"# Notes for pkce extension","ref":"pkce.html","title":"Notes for pkce extension","type":"extras"},{"doc":"There are a few pre-requisites.\n\nHave a way to inject `:current_user` in the conn, otherwise implement it before continuing, also keep in mind that in most cases the `:current_user` is the same as the resource owner on OAuth.\n\nOn our server, we need to create a client with pkce value as true.\n\n```elixir\nalias Boruta.Ecto.Admin.Clients\n{:ok, client} = %{authorization_code_ttl: 60, access_token_ttl: 60 * 60, pkce: true, name: \"My Awesome Client App\", redirect_uris: [\"http://localhost:4000\"]} |> Clients.create_client()\n```\n\nWe will get the `client.id` among others fields.","ref":"pkce.html#prerequisites","title":"Prerequisites - Notes for pkce extension","type":"extras"},{"doc":"From the client perspective, who is in charge of sending the `code_challenge` and `code_challenge_method` among other fields during the request of the code, there is a `code_verifier` variable which is a key concept to keep in mind because `code_challenge` depends on it and also depends on `code_challenge_method`, let's see an example.\n\nIn our client, we must have an URL to request the `/oauth/authorize` endpoint to get the code. To build the URL we should follow a similar approach to the proposed.\n```elixir\nclient_id = client.id\ncode_verifier = \"a-strong-random-string\"\nstate = \"a-random-verifiable-state-on-client\"\n# When the code challenge method is S256, we must follow the standard code_challenge = BASE64URL-ENCODE(SHA256(ASCII(code_verifier)))\ncode_challenge_method = \"S256\"\ncode_challenge = :crypto.hash(:sha256, code_verifier) |> Base.url_encode64(padding: false)\n# Only if our client does not support sha256 crypto it goes through the plain method. Then code_challenge = code_verifier\n#\n# Keep in mind that it must be used only if your architecture does not support Sha256 crypto. otherwise use it.\n#\n# code_challenge_method = \"plain\"\n# code_challenge = code_verifier\n\n# Then we are ready to build your URL.\n\nurl = \"http://localhost:4000/oauth/authorize?response_type=code&client_id=#{client_id}&redirect_uri=http://localhost:4000&state=#{state}&code_challenge=#{code_challenge}&code_challenge_method=#{code_challenge_method}\"\n\n\"http://localhost:4000/oauth/authorize?response_type=code&client_id=client-id&redirect_uri=http://localhost:4000&state=a-random-verifiable-state-on-client&code_challenge=PdSbV6nNRlruDAqmULREHO_pLwfmNnNGA-HGIjmc6VA==&code_challenge_method=S256\"\n\n```\n\nYou can paste the URL in your browser with a logged user and retrieve the code from the response URL.\n\nHaving the code the next step is to retrieve a token, in this case, the client must call the `oauth/token` to get it.\n\nBased on the previous step, a request example could look like this.\n\n```bash\ncurl --location --request POST 'http://localhost:4000/oauth/token' \\\n--header 'Content-Type: application/x-www-form-urlencoded' \\\n--data-urlencode 'grant_type=authorization_code' \\\n--data-urlencode 'client_id=c96951c5c-f7b5-4d03-ac09-657833d113df' \\\n--data-urlencode 'code=4vlPc010GKutGlCYeT3VZFbyoVewmtNPOo0c5DXvt1iQ9pBd0BkeCutVwHWFNq8vzdTfIPppFGlhuYuOIWtvnU' \\\n--data-urlencode 'code_verifier=a-strong-random-string' \\\n--data-urlencode 'redirect_uri=http://localhost:4000' \\\n--data-urlencode 'state=a-random-verifiable-state-on-client'\n```\n\nIt's important to notice that the code_verifier has the value previously generated and is not encrypted. This is because the server knows that you use encrypted method S256,\nand it implements the logic to handle it.\n\nAnd that is all. If your configuration is right, you will get a response like this.\n\n```json\n{\"access_token\":\"BHYQabRwti673RVhoMnqXtrvgiEilawgaAj1KosrQV9pGQ4DBENXzhUHeSjZE4K02S8bNtlNL2gU9bzFhKMoQK\",\"expires_in\":3600,\"refresh_token\":\"r90e4S9uaqH59xMO809Ws2uzWPtcaMlcF1J7fcaaSruE1AAc30WuykNAL73GGNECtmYezQSf2CzJGgQWPqdM5M\",\"token_type\":\"bearer\"}\n```","ref":"pkce.html#flow-steps","title":"Flow steps - Notes for pkce extension","type":"extras"},{"doc":"# Notes about confidential clients\n\nThis server manage confidential clients as stated in [OAuth 2.0 RFC](https://datatracker.ietf.org/doc/html/rfc6749). Clients have a `confidential` attribute as a boolean to state if the client is set to be confidential. Confidential clients enforce check of client secret in some cases. Here is described how this implementation manage them for each flow:\n- __Client Credentials__ - always enforces check of client secret\n- __Authorization Code Grant__ - enforces check of client secret only for confidential clients on access token request, does not check client secret during authorization phase\n- __Hybrid Flow__ - has the same behavior as the authorization code grant\n- __Implicit Grant__ - does not check client secret\n- __Resource Owner Password Credentials__ - enforces check of client secret only for confidential clients\n- __Refresh Token__ - always enforces check of client secret, `public_refresh_token` overrides the confidentiality\n- __Introspect__ - always enforces check of client secret\n- __Revoke__ - always enforces check of client secret, `public_revoke` overrides the confidentiality","ref":"confidential_clients.html","title":"Notes about confidential clients","type":"extras"},{"doc":"# Migration from 1.X\n\nVersion 2 of Boruta brings OpenID Connect, several changes were made in order to stick to the specification:","ref":"migration.html","title":"Migration from 1.X","type":"extras"},{"doc":"`Boruta.Oauth.AuthorizeResponse` and `Boruta.Oauth.TokenResponse` do not provide token value in `value` field but prefer giving value by token type `code`, `access_token` or `id_token`.\n\n```elixir\n%AuthorizeResponse{\n   type: \"code\",\n   value: value,\n   expires_in: 60\n}\n```\n\nbecomes\n\n```elixir\n%AuthorizeResponse{\n   type: :code,\n   code: value,\n   expires_in: 60\n}\n```","ref":"migration.html#oauth-response-structs-changes","title":"OAuth response structs changes - Migration from 1.X","type":"extras"},{"doc":"`boruta.gen.migration` task has been updated. Running the task will create required migrations to upgrade database schemas according to the updated `Ecto.Schema` modules.\n\nIn order to have the schema up to date, you need to generate migrations and run them\n\n```\nmix boruta.gen.migration\nmix ecto.migrate\n```","ref":"migration.html#database-migrations","title":"Database migrations - Migration from 1.X","type":"extras"},{"doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html) according to OAuth / OpenID connect specifications, changes may break in order to comply with those.\n\n## [2.3.2] - 2023-12-23\n\n- Note that you must run the `boruta.gen.migration` task to keep your database schema up to date while upgrading to this version.\n\n#","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"- clients have a `metadata` attribute where one can store json objects\n- according to OpenID Connect core 1.0, clients have a `logo_uri` attribute\n- `boruta.gen.migration` triggers a form to get statistics about boruta usage\n\n## [2.3.1] - 2023-04-24\n\n- Note that you must run the `boruta.gen.migration` task to keep your database schema up to date while upgrading to this version.\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"- public key is optional for oauth clients\n\n## [2.3.0] - 2023-04-09\n\n- Note that you must run the `boruta.gen.migration` task to keep your database schema up to date while upgrading to this version.\n\n#","ref":"changelog.html#fixed","title":"Fixed - Changelog","type":"extras"},{"doc":"- configuration and support for client authentication methods (`client_secret_post`, `client_secret_basic`, `client_secret_jwt`, `private_key_jwt`) [RFC 7521](https://www.rfc-editor.org/rfc/rfc7521), [RFC 7523](https://www.rfc-editor.org/rfc/rfc7523)\n- dynamic client registration support [OpenID Connect Dynamic Client Registration 1.0 incorporating errata set 1](https://openid.net/specs/openid-connect-registration-1_0.html)\n- handle userinfo signed responses\n- client key pair regeneration admin function `Admin.regenerate_client_key_pair/1,3`\n\n## [2.2.2] - 2022-10-25\n\n- Note that you must run the `boruta.gen.migration` task to keep your database schema up to date while upgrading to this version.\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"- allow lower case bearer authorization header\n- prompt=none management for authorization code grant requests\n- store the previous code associated with the delivered access token in authorization code grants\n- prompt=none management for authorization code grant requests\n\n## [2.2.1] - 2022-10-16\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"- remove symmetric keys from openid jwks\n\n## [2.2.0] - 2022-09-13\n\n- Note that you must run the `boruta.gen.migration` task to keep your database schema up to date while upgrading to this version.\n- Upgrade to this version need you to invalidate the cache by running `Boruta.Config.cache_backend().delete_all()`\n\n#","ref":"changelog.html#security","title":"Security - Changelog","type":"extras"},{"doc":"- confidential client management as stated in OAuth 2.0 RFC, documented [here](guides/confidential_clients.md). It defaults to false (already existing clients will not be confidential)\n- token as the created `Boruta.Oauth.Token` attribute in `Boruta.Oauth.TokenResponse`\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"- client credentials does not check client secret by default anymore, the client has to be set as confidential to do so\n\n#","ref":"changelog.html#changed","title":"Changed - Changelog","type":"extras"},{"doc":"- enable refresh token rotation, revoke previous refresh token on successful refresh token requests\n- fix redirect_uri injection in implicit, hybrid and code grants\n\n## [2.1.5] - 2022-06-15\n\n#","ref":"changelog.html#security","title":"Security - Changelog","type":"extras"},{"doc":"- id tokens include `kid` header with the corresponding client id\n- `Boruta.Oauth.ResourceOwner` `extra_claims` attribute that defines claims to be included in id tokens\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"- do not not issue an access token in authorization code and hybrid grants if code was issued to an other client\n\n#","ref":"changelog.html#security","title":"Security - Changelog","type":"extras"},{"doc":"- id token `at_hash` and `c_hash` binary sizes for SHA256 and SHA384 signature hash algorithms\n\n## [2.1.4] - 2022-06-07\n\nNote that you must run the `boruta.gen.migration` task to keep your database schema up to date while upgrading to this version.\n\n#","ref":"changelog.html#fixed","title":"Fixed - Changelog","type":"extras"},{"doc":"- handle `response_mode` in hybrid requests\n- client id_token validation for ecto adapter\n- per client id token signature algorithm configuration (introduce a database schema change)\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"- prefer `invalid_grant` to `invalid_code` and `invalid_refresh_token`\n- error messages have been improved\n\n## [2.1.3] - 2022-05-17\n\n#","ref":"changelog.html#changed","title":"Changed - Changelog","type":"extras"},{"doc":"- handle `response_mode` in hybrid requests errors\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"- respond to authorize requests with `token_type` only when an access token is returned\n- generated migrations can be rollbacked\n- clients pkey constraint do not crash on admin create\n\n## [2.1.2] - 2022-05-02\n\n#","ref":"changelog.html#fixed","title":"Fixed - Changelog","type":"extras"},{"doc":"- hybrid requests shall return all errors as fragment\n\n## [2.1.1] - 2022-04-30\n\n#","ref":"changelog.html#fixed","title":"Fixed - Changelog","type":"extras"},{"doc":"- dialyzer warning on `Boruta.Oauth.Error` struct type\n\n## [2.1.0] - 2022-04-29\n\n#","ref":"changelog.html#fixed","title":"Fixed - Changelog","type":"extras"},{"doc":"- OpenID Connect jwks endpoint domain and application layer generation\n- OpenID Connect userinfo endpoint domain and application layer generation\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"- OpenID Connect prompt=none login_required errors in domain\n\n## [2.0.1] - 2022-04-12\n\n#","ref":"changelog.html#fixed","title":"Fixed - Changelog","type":"extras"},{"doc":"- expose `Boruta.Oauth.Client.grant_types/0`\n- expose `Boruta.Oauth.IdToken.signature_alg/0` and `Boruta.Oauth.IdToken.hash_alg/0`\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"- `Admin.delete_inactive_tokens/0,1` does not return deleted tokens\n\n#","ref":"changelog.html#changed","title":"Changed - Changelog","type":"extras"},{"doc":"- generated openid authorize controller prompt=none error params type\n\n## [2.0.0] - 2022-01-26\n\n#","ref":"changelog.html#fixed","title":"Fixed - Changelog","type":"extras"},{"doc":"- `Boruta.Ecto.Admin.get_scopes_by_names/1`\n- `Boruta.Ecto.Admin.regenerate_client_secret/1,2`\n- `Boruta.Ecto.Admin.delete_inactive_tokens/0,1`\n- `Boruta.Ecto.Client.grant_types/0`\n- ability to insert/update clients with given id/secret\n- inserting/updating a client inserts non existing authorized_scopes\n- `oauth_module` injection in `boruta.gen.controllers` generated controllers default to `Boruta.Oauth`\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"- store previous token while refreshing access tokens (need to run `boruta.gen.migration` mix task to be up to date)\n- `Boruta.Ecto.Admin.list_active_tokens/0,1` returns query result instead of an `Ecto.Query`\n- `Boruta.Oauth.ResourceOwners.claims/2` callback takes a `Boruta.Oauth.ResourceOwner` struct instead of `sub` as parameter\n\n## [2.0.0-rc.1] - 2021-11-17\n\n#","ref":"changelog.html#changed","title":"Changed - Changelog","type":"extras"},{"doc":"- better Ecto errors management\n- remove padding from pkce code challenge checks\n- reduce resource_owners adapter calls\n\n#","ref":"changelog.html#fixed","title":"Fixed - Changelog","type":"extras"},{"doc":"- domain wildcard for client redirect_uris\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"- Ecto `ClientsAdapter.get_by(id: id, secret: secret)` is removed in preference of `ClientsAdapter.get_client(id)` and `Oauth.Client.check_secret(client, secret)`.\n- Ecto `ClientsAdapter.get_by(id: id, redirect_uri: redirect_uri)` is removed in preference of `ClientsAdapter.get_client(id)` and `Oauth.Client.check_redirect_uri(client, redirect_uri)`.\n\n## [2.0.0-rc.0] 2021-10-12\n\n#","ref":"changelog.html#removed","title":"Removed - Changelog","type":"extras"},{"doc":"- [OpenID Connect core 1.0](https://openid.net/specs/openid-connect-core-1_0.html) integration\n  - hybrid flow\n  - authorization code and implicit grants with OpenID Connect compatibility\n- `public_revoke` per client configuration allowing to revoke tokens without providing client secret.\n- `introspect` and `revoke` supported grant types per client configuration.\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"- `Boruta.Oauth.AuthorizeResponse` and `Boruta.Oauth.TokenResponse` do not provide token value in `value` field but prefer giving value by token type `code`, `access_token` or `id_token`.\n```\n%AuthorizeResponse{\n   type: \"code\",\n   value: value,\n   expires_in: 60\n}\n```\nbecomes\n```\n%AuthorizeResponse{\n   type: :code,\n   code: value,\n   expires_in: 60\n}\n```\n- add nonce column to tokens\n- default column values migrations\n- migration management `boruta.gen.migrations` does incremental changes\n\n#","ref":"changelog.html#changed","title":"Changed - Changelog","type":"extras"},{"doc":"- codes are revoked after first usage\n\n#","ref":"changelog.html#security","title":"Security - Changelog","type":"extras"},{"doc":"- `boruta.gen.controllers` generated paths in umbrella apps\n\n## [1.2.1] - 2021-10-10\n\n#","ref":"changelog.html#fixed","title":"Fixed - Changelog","type":"extras"},{"doc":"- remove redirect_uris regex pattern check\n\n## [1.2.0] - 2021-09-15\n\n#","ref":"changelog.html#security","title":"Security - Changelog","type":"extras"},{"doc":"- `public_refresh_token` per client configuration allowing to refresh tokens without providing client secret.\n- `refresh_token_ttl` per client configuration setting refresh tokens duration (along with `refresh_tokne_max_ttl` :boruta mix configuration).\n- `issuer` :boruta mix configuration.\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"- `boruta.gen.controllers` generated paths in umbrella apps.\n- Refreshed tokens has associated access_token scope as default.\n- Requests with no client secret won't raise an error.\n\n#","ref":"changelog.html#fixed","title":"Fixed - Changelog","type":"extras"},{"doc":"- `invalid_client` do not return neither format, nor redirect_uri in `Boruta.Oauth.Error`.\n\n## [1.1.0] - 2021-08-16\n\n#","ref":"changelog.html#changed","title":"Changed - Changelog","type":"extras"},{"doc":"- `AuthorizeApplication`, `IntrospectApplication`, `RevokeApplication`, and `TokenApplication` behaviours allowing to implement separately different OAuth use cases.\n- `list_active_tokens` Ecto admin function\n- `Boruta.AccessTokensAdapter`, `Boruta.CodesAdapter`, `Boruta.ClientsAdapter`, and `Boruta.ScopesAdapter` encapsulating adapters that are set in configuration.\n- `Boruta.Oauth.AuthorizeResponse.redirect_to_url/1` function\n- `Boruta.Oauth.Error.redirect_to_url/1` function\n- `boruta.gen.controllers` mix task\n- `Boruta.Ecto` schemas documentation\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"- do not issue access_tokens from other clients refresh tokens\n\n#","ref":"changelog.html#security","title":"Security - Changelog","type":"extras"},{"doc":"- Internal server errors when no client_id provided to token and refresh_token grants\n\n## [1.0.3] - 2021-07-29\n\n#","ref":"changelog.html#fixed","title":"Fixed - Changelog","type":"extras"},{"doc":"- Refresh token revocation\n\n## [1.0.2] - 2021-06-29\n\n#","ref":"changelog.html#security","title":"Security - Changelog","type":"extras"},{"doc":"- Different OAuth flows integration guides\n\n## [1.0.1] - 2021-03-10\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"- Migration fix generated by `mix boruta.gen.migration` task\n\n#","ref":"changelog.html#fixed","title":"Fixed - Changelog","type":"extras"},{"doc":"- Documentation\n\n## [1.0.0] - 2021-03-10","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"}]}